@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -557,7 +557,7 @@ void AI::CommandUnitTransport( Unit *u, AIObj &obj ) {
         Point dest = FollowPath( t, tp, 1 );
         Gam->MoveUnit( t, dest );
         moved = true;
-      } else tp.Clear();
+      }
     }
 
     if ( moved ) {
@@ -598,7 +598,6 @@ bool AI::CommandUnitRepair( Unit *u ) {
       }
       return true;
     } else last = b;
-    path.Clear();
   } while ( b );
   return false;
 }
@@ -844,17 +843,16 @@ Unit *AI::FindBestTarget( const Unit *u ) {
 
   if ( u->IsDefensive() ) return NULL;
 
+  Path path( map );
   for ( Unit *tg = static_cast<Unit *>( mission.GetUnits().Head() );
         tg; tg = static_cast<Unit *>( tg->Next() ) ) {
     if ( (tg->Owner() == &mission.GetOtherPlayer(*player)) && u->CanHitType( tg )
          && !tg->IsSheltered() ) {
 
       if ( UnitCanReach( u, tg->Position(), u->WeaponRange(tg) ) ) {
-        Path path( map );
         unsigned short val = 0;
         short cost = path.Find( u, u->Position(), tg->Position(),
                                 PATH_FAST, u->WeaponRange( tg ) );
-        path.Clear();
 
         if ( cost >= 0 ) {
           val = MAX( 0, 10000 - UnitStrength( tg ) - cost * 50 );
@@ -903,7 +901,7 @@ Point AI::FindBestHex( const Unit *u, const Unit *enemy ) const {
   // for long-range attacks, terrain type etc. are not important
   if ( (turns > 1) || ((u->WeaponRange(enemy) > 1) && (turns >= 0)) )
     hex = FollowPath( u, path );
-  else if ( turns == 1 ) {
+  else if ( turns >= 0 ) {
     Unit *sup;
     Point nb[6], hlp;
     short vals[6], bestval = _CF_BEST_HEX_INVALID;
@@ -912,10 +910,10 @@ Point AI::FindBestHex( const Unit *u, const Unit *enemy ) const {
     map->GetNeighbors( enemy->Position(), nb );
     for ( i = NORTH; i <= NORTHWEST; ++i ) {
       if ( (nb[i].x != -1) && (nb[i].y != -1) && !map->GetUnit( nb[i] ) &&
+           ((map->TerrainTypes( nb[i] ) & TT_ENTRANCE) == 0) &&
            (path.Find( u, u->Position(), nb[i], PATH_FAST ) == 1) ) {
-        vals[i] = 0;
+        vals[i] = -Distance( u->Position(), nb[i] );
       } else vals[i] = _CF_BEST_HEX_INVALID;
-      path.Clear();
     }
 
     for ( i = NORTH; i <= NORTHWEST; ++i ) {
@@ -991,9 +989,7 @@ Transport *AI::FindTransport( const Unit *u, const Point &dest,
               booked = true;
             }
           }
-          p.Clear();
         }
-        tp.Clear();
       }
     }
   }
@@ -1010,9 +1006,9 @@ Transport *AI::FindTransport( const Unit *u, const Point &dest,
         if ( to->type == AI_OBJ_TRANSPORT ) continue;
       }
 
-      TransPath p1( map, t ), p2( map, t );
-      if ( (p1.Find( u, t->Position(), u->Position(), PATH_FAST, dist ) != -1) &&
-           (p2.Find( u, t->Position(), dest, PATH_FAST, dist ) != -1) ) {
+      TransPath tp2( map, t );
+      if ( (tp2.Find( u, t->Position(), u->Position(), PATH_FAST, dist ) != -1) &&
+           (tp2.Find( u, t->Position(), dest, PATH_FAST, dist ) != -1) ) {
         // both hexes accessible -> ok
         // prefer these transports over those found above as this way we
         // might get a private one all for ourselves
@@ -1064,7 +1060,6 @@ bool AI::UnitGoTo( Unit *u, const Point &dest, unsigned short dist ) {
     short turns;
     Transport *t = FindTransport( u, dest, dist, true );
     if ( t ) {
-      p.Clear();
       turns = p.Find( u, u->Position(), t->Position() );
       if ( (turns != 1) && (t->UnitCount() == 0) ) {
         // get the transport closer first (only if it's empty)
@@ -1075,7 +1070,6 @@ bool AI::UnitGoTo( Unit *u, const Point &dest, unsigned short dist ) {
           Gam->MoveUnit( t, end );
           Gam->SelectUnit( u );
 
-          p.Clear();
           turns = p.Find( u, u->Position(), end );
           if ( turns != -1 ) {
             end = FollowPath( u, p );
