@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -55,6 +55,7 @@ extern Options CFOptions;
 #define G_BUTTON_UNIT_INFO	 22
 #define G_BUTTON_UNIT_CONTENT	 23
 #define G_BUTTON_UNIT_SWEEP	 24
+#define G_BUTTON_UNIT_UNDO	 25
 
 #define G_KEY_QUIT		0
 #define G_KEY_INFO		1
@@ -127,6 +128,7 @@ void Game::InitWindows( void ) {
     mwin->GetMapView()->SetMap( &mission->GetMap() );	// attach map to map window
     shader = new MoveShader( &mission->GetMap(), mission->GetUnits(),
                              mwin->GetMapView()->GetFogBuffer() );
+    ExecPreStartEvents();
   }
 }
 
@@ -246,7 +248,7 @@ int Game::Save( const char *filename ) {
   if ( fname.length() == 0 ) return -1;
  
   File file( fname );
-  if ( !file.Open( "wb" ) ) {
+  if ( !file.Open( "wb", false ) ) {
     new NoteWindow( MSG(MSG_ERROR), MSG(MSG_ERR_WRITE), 0, view );
     return -1;
   }
@@ -256,10 +258,12 @@ int Game::Save( const char *filename ) {
   unsigned int num = last_file_name.rfind( '.' );
   last_file_name.erase( num );
 
+  unsigned short mflags = mission->GetFlags();
+  mission->SetFlags( mflags|GI_SAVEFILE );
   mission->Save( file );
   file.Close();
 
-  if ( mission->GetFlags() & GI_PBEM ) {
+  if ( mflags & GI_PBEM ) {
     string msg = StringUtil::strprintf( MSG(MSG_GAME_SAVED_PBEM), fname );
     NoteWindow *nw = new NoteWindow( MSG(MSG_GAME_SAVED), msg, 0, view );
     nw->SetButtonID( 0, G_BUTTON_SHUTDOWN );
@@ -274,7 +278,7 @@ int Game::Save( const char *filename ) {
 // DESCRIPTION: Load a game from a mission file (start a new game) or
 //              a save file (resume game).
 // PARAMETERS : filename - data file name
-// RETURNS    : 0 on success, non-zero otherwise
+// RETURNS    : 0 on success, -1 otherwise
 ////////////////////////////////////////////////////////////////////////
 
 int Game::Load( const char *filename ) {
@@ -293,20 +297,18 @@ int Game::Load( const char *filename ) {
     mission->SetLocale( CFOptions.GetLanguage() );
 
     unsigned short flags = mission->GetFlags();
-    // if this is a new map (GI_SAVEFILE not set) GI_AI and
-    // GI_CAMPAIGN only serve informational purposes
+    // if this is a new map (GI_SAVEFILE not set) GI_SKIRMISH,
+    // GI_CAMPAIGN, and GI_AI only serve as information
     if ( !(flags & GI_SAVEFILE) ) {
-      flags &= ~(GI_AI|GI_CAMPAIGN);
-      flags |= (GI_SAVEFILE|
-               (CFOptions.IsPBEM() ? GI_PBEM : 0)|
+      flags &= ~(GI_AI|GI_CAMPAIGN|GI_SKIRMISH);
+      flags |= ((CFOptions.IsPBEM() ? GI_PBEM : 0)|
                (CFOptions.GetCampaign() ? GI_CAMPAIGN : 0));
     }
     mission->SetFlags( flags );
-
     rc = 0;
   } else {
     delete mission;
-    mission = NULL; 
+    mission = NULL;
   }
 
   return rc;
@@ -388,6 +390,8 @@ GUI_Status Game::StartTurn( void ) {
       return GUI_RESTART;
   }
 
+  undo.Disable();
+
   if ( mission->GetPhase() == TURN_START ) {
     // replay
     History *history = mission->GetHistory();
@@ -411,17 +415,22 @@ GUI_Status Game::StartTurn( void ) {
 
     mission->SetPhase( TURN_IN_PROGRESS );
     mv->SetCursorImage( IMG_CURSOR_IDLE );
-    CheckEvents();
   }
 
   if ( !player.IsHuman() ) {
+    CheckEvents();
     AI ai( *mission );
     ai.Play();
     EndTurn();
   } else {
+
+    mv->Enable();
+    mv->SetFlags( MV_DIRTY );
+    CheckEvents();
+    mv->UnsetFlags( MV_DIRTY );
+
     // set the cursor to one of the player's units
     Point startcursor( 0, 0 );
-
     for ( Unit *u = static_cast<Unit *>(mission->GetUnits().Head());
           u; u = static_cast<Unit *>(u->Next()) ) {
       if ( u->Owner() == &player ) {
@@ -431,7 +440,6 @@ GUI_Status Game::StartTurn( void ) {
     }
 
     view->DisableUpdates();
-    mv->Enable();
     mv->CenterOnHex( startcursor );
     SetCursor( startcursor );
     view->EnableUpdates();
@@ -453,7 +461,7 @@ bool Game::SetPlayerPassword( Player *player ) const {
   const char *pw, *msg1 = MSG(MSG_CHOOSE_PASSWORD);
 
   pwin = new PasswordWindow( player->Name(), msg1,
-                             player->Password(), view );
+                             player->Password(), false, view );
 
   do {
     pwin->EventLoop();
@@ -504,7 +512,7 @@ bool Game::SetPlayerPassword( Player *player ) const {
 
 bool Game::CheckPassword( const char *title, const char *msg,
                           const char *pw, short retry ) const {
-  PasswordWindow *pwin = new PasswordWindow( title, msg, pw, view );
+  PasswordWindow *pwin = new PasswordWindow( title, msg, pw, true, view );
   bool pw_ok = false;
 
   do {
@@ -533,28 +541,51 @@ bool Game::CheckPassword( const char *title, const char *msg,
 
 GUI_Status Game::CheckEvents( void ) {
   Event *e = static_cast<Event *>( mission->GetEvents().Head() ), *e2;
-  MessageWindow *win = NULL;
 
   while ( e ) {
     e2 = static_cast<Event *>( e->Next() );
-    short check = e->Check();
-    if ( check > 0 ) {
+    if ( e->Discarded() ) {
+      e->Remove();
+      delete e;
+    } else if ( e->Check() ) {
       // event will be executed and can be taken out of the queue
       e->Remove();
-
-      // check == 2 means we need a window to display a message
-      if ( (check == 2) && !win ) win = new MessageWindow( "", NULL, view );
-      e->Execute( win );
+      e->Execute( view );
       delete e;
 
-      if ( check == 2 ) win->EventLoop();
+      // after events have been triggered, undo is no longer allowed
+      undo.Disable();
     }
     e = e2;
   }
-  if ( win ) view->CloseWindow( win );
   return GUI_OK;
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : Game::ExecPreStartEvents
+// DESCRIPTION: This method is called before the very first turn. It is
+//              used to execute initial ETRIGGER_HANDICAP events without
+//              affecting the turn history (this means e.g. that units
+//              created due to handicaps to not appear in turn replays).
+// PARAMETERS : -
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void Game::ExecPreStartEvents( void ) {
+  if ( (mission->GetTime() == 0) && (mission->GetPhase() == TURN_START) ) {
+    Event *e = static_cast<Event *>( mission->GetEvents().Head() ), *e2;
+    while ( e ) {
+      e2 = static_cast<Event *>( e->Next() );
+      if ( (e->Trigger() == ETRIGGER_HANDICAP) && e->Check() ) {
+        e->Remove();
+        e->Execute( view );
+        delete e;
+      }
+      e = e2;
+    }
+  }
+}
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : Game::HaveWinner
 // DESCRIPTION: Check whether one of the players has won completed his
@@ -627,9 +658,8 @@ GUI_Status Game::EndTurn( void ) {
       if ( !u->IsAlive() ) {
         u->Remove();
         delete u;
-      } else {
-        if ( u->Owner() == &p ) u->RefreshMoves();
-        else u->UnsetFlags( U_MOVED|U_ATTACKED|U_DONE|U_BUSY );
+      } else if ( u->Owner() != &p ) {
+        u->UnsetFlags( U_MOVED|U_ATTACKED|U_DONE|U_BUSY );
       }
       u = next;
     }
@@ -791,21 +821,14 @@ Unit *Game::MoveUnit( Unit *u, const Point &dest ) {
     SoundEffect *sfx = u->MoveSound();
     if ( sfx ) sfx->Play( Audio::SFX_LOOP );
 
-    if ( !u->IsSheltered() ) {
-      map->SetUnit( NULL, pos );
-      mv->UpdateHex( pos );
-    } else {
-      Transport *t = static_cast<Transport *>( map->GetUnit( pos ) );
-      if ( t ) t->RemoveUnit( u );
-      else map->GetBuilding( pos )->RemoveUnit( u );
-    }
+    undo.Register( u );
+    RemoveUnit( u );
 
     short step, oldhex = map->HexTypeID( dest ); 
     while ( (step = path.GetStep( u->Position() )) != -1)
       MoveUnit( u, (Direction)step );
 
     u->SetFlags( U_MOVED );
-    u->SetMoves( 0 );
 
     if ( sfx ) sfx->Stop();
 
@@ -825,7 +848,6 @@ Unit *Game::MoveUnit( Unit *u, const Point &dest ) {
       return NULL;
     }
 
-    shader->Clear();
     shader->ShadeMap( u );
     if ( mv->CursorEnabled() ) mwin->GetPanel()->Update( u );
     mwin->Draw();
@@ -850,13 +872,7 @@ int Game::MoveUnit( Unit *u, Direction dir ) {
   Point posnew;
   if ( map->Dir2Hex( pos, dir, posnew ) ) return -1;
 
-  short cost, moves = u->Moves();
-  if ( u->IsAircraft() || u->IsMine() || map->GetUnit( posnew ) ) cost = MCOST_MIN;
-  else cost = map->MoveCost( posnew );
-  if ( cost > moves ) return -1;
-
   u->Face( dir );
-  u->SetMoves( moves - cost );
 
   if ( mv->Enabled() &&
        (mv->HexVisible(pos) || mv->HexVisible(posnew)) )
@@ -871,6 +887,28 @@ int Game::MoveUnit( Unit *u, Direction dir ) {
   return 0;
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : Game::RemoveUnit
+// DESCRIPTION: Remove a unit from the map.
+// PARAMETERS : u  - unit to be removed
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void Game::RemoveUnit( Unit *u ) {
+  Map &map = mission->GetMap();
+
+  if ( !u->IsSheltered() ) {
+    map.SetUnit( NULL, u->Position() );
+    mwin->GetMapView()->UpdateHex( u->Position() );
+  } else {
+    MapObject *o = map.GetMapObject( u->Position() );
+    if ( o ) {
+      if ( o->IsUnit() ) static_cast<Transport *>( o )->RemoveUnit( u );
+      else static_cast<Building *>(o)->RemoveUnit( u );
+    }
+  }
+}
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : Game::UnitTargets
 // DESCRIPTION: Find out whether the unit still has things to do, i.e.
@@ -954,7 +992,6 @@ void Game::SelectUnit( Unit *u ) {
     if ( mv->Enabled() ) {
       Audio::PlaySfx( Audio::SND_GAM_SELECT, 0 );
       mv->EnableFog();
-      shader->Clear();
       shader->ShadeMap( unit );
       mwin->Draw();
       mwin->Show();
@@ -1037,7 +1074,6 @@ void Game::EnterSpecialMode( unsigned char mode ) {
     MinesweeperShader mss( &mission->GetMap(), mission->GetUnits(), mv->GetFogBuffer() );
 
     mv->EnableFog();
-    mss.Clear();
     mss.ShadeMap( unit );
     mwin->Draw();
     mwin->Show();
@@ -1068,15 +1104,45 @@ void Game::ClearMine( Transport *sweeper, Unit *mine ) {
   } else allow = sweeper->Allow( mine );
 
   if ( allow ) {
-    mine->SetMoves( MCOST_MIN );
     MoveUnit( mine, spos );
 
-    if ( !UnitTargets(sweeper) ) DeselectUnit();
+    if ( !MinesweeperTargets(sweeper) ) {
+      sweeper->SetFlags( U_DONE );
+      DeselectUnit();
+    }
+
+    undo.Disable();
 
   } else new NoteWindow( MSG(MSG_ERROR), MSG(MSG_ERR_SWEEPER_FULL),
                          WIN_CLOSE_ESC, view );
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : Game::Undo
+// DESCRIPTION: Undo the last move command the player issued.
+// PARAMETERS : -
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void Game::Undo( void ) {
+  if ( unit ) DeselectUnit();
+
+  Map &map = mission->GetMap();
+  Unit *u = undo.GetUnit();
+
+  RemoveUnit( u );
+  u->Face( undo.GetDirection() );
+  u->UnsetFlags( U_DONE|U_MOVED );
+  map.SetUnit( u, undo.GetPosition() );
+  mwin->GetMapView()->UpdateHex( undo.GetPosition() );
+  mwin->Show();
+
+  if ( mission->GetHistory() )
+    mission->GetHistory()->EraseMoveEvents( *u );
+
+  undo.Disable();
+}
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : Game::HandleEvent
 // DESCRIPTION: Handle key and mouse button events special to the map
@@ -1230,6 +1296,7 @@ void Game::SelectCommand( const Point &hex ) {
       } else if ( unit->CanHit( u ) ) {   // attack the unit
         mission->RegisterBattle( unit, u );
         DeselectUnit();
+        undo.Disable();
       }
     } else MoveUnit( unit, hex );  // try to move there
     break;
@@ -1237,8 +1304,10 @@ void Game::SelectCommand( const Point &hex ) {
   case MODE_IDLE:
     if ( u && (u->Owner() == &p) ) SelectUnit( u );
     else {
-      Building *b = map->GetBuilding( hex );
-      if ( b ) ContainerContent( b );
+      Building *b;
+
+      if ( map->IsShop( hex ) && (b = map->GetBuilding( hex )) )
+        ContainerContent( b );
     }
     break;
 
@@ -1477,7 +1546,7 @@ void Game::GameMenu( void ) {
 void Game::UnitMenu( Unit *u ) {
 
   if ( (u->Owner() == &mission->GetPlayer()) &&
-       (u->IsTransport() || u->IsMinesweeper()) ) {
+       (u->IsTransport() || u->IsMinesweeper() || (undo.GetUnit() == u)) ) {
     MenuWindow *menu = new MenuWindow( u->Name(), this, view );
 
     menu->AddItem( 0, G_BUTTON_UNIT_INFO, 0, MSG(MSG_B_UNIT_INFO) );
@@ -1489,6 +1558,9 @@ void Game::UnitMenu( Unit *u ) {
       menu->AddItem( 0, G_BUTTON_UNIT_SWEEP,
             (MinesweeperTargets( u ) ? 0 : WIDGET_DISABLED), MSG(MSG_B_UNIT_SWEEP) );
 
+    if ( undo.GetUnit() == u )
+      menu->AddItem( 0, G_BUTTON_UNIT_UNDO, 0, MSG(MSG_B_UNIT_UNDO) );
+
     menu->Layout();
     g_tmp_prv_unit = u;
   } else UnitInfo( u );
@@ -1604,6 +1676,10 @@ GUI_Status Game::WidgetActivated( Widget *button, Window *win ) {
     }
     EnterSpecialMode( MODE_SWEEP );
     break;
+  case G_BUTTON_UNIT_UNDO:
+    view->CloseWindow( win );
+    Undo();
+    break;
   }
 
   return rc;
