@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -46,6 +46,25 @@ extern class Image *Images[];
 
 #define DEFAULT_DELAY	(5 * ANIM_SPEED_UNIT)
 
+class UndoCache {
+public:
+  UndoCache( void ) : unit(NULL) {}
+
+  void Disable( void ) { unit = NULL; }
+  void Register( Unit *u )
+    { unit = u; pos = u->Position(); dir = u->Facing(); }
+
+  bool Disabled( void ) const { return unit == NULL; }
+  Unit *GetUnit( void ) const { return unit; }
+  const Point &GetPosition( void ) const { return pos; }
+  unsigned char GetDirection( void ) const { return dir; }
+
+private:
+  Unit *unit;
+  Point pos;
+  unsigned char dir;
+};
+
 class Game : public WidgetHook {
 public:
   Game( View *view );
@@ -79,6 +98,7 @@ private:
   void ClearMine( Transport *sweeper, Unit *mine );
   bool HaveWinner( void );
   GUI_Status CheckEvents( void );
+  void ExecPreStartEvents( void );
 
   void MoveCommand( int key );
   void SelectCommand( const Point &hex );
@@ -86,6 +106,8 @@ private:
   void HandleLMB( const Point &hex );
   void EnterSpecialMode( unsigned char mode );
   void SelectNextUnit( void );
+  void RemoveUnit( Unit *u );
+  void Undo( void );
 
   bool UnitTargets( Unit *u ) const;
   bool MinesweeperTargets( Unit *u ) const;
@@ -113,6 +135,8 @@ private:
   MoveShader *shader;
   View *view;
 
+  UndoCache undo;
+
   string last_file_name; // remember save file names
   Unit *g_tmp_prv_unit;
 };
