@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -75,6 +75,53 @@ InitWindow::InitWindow( View *view, Window *title ) :
   view->SetFGPen( Color( 0x00d87c00 ) );
   view->SetBGPen( Color( CF_COLOR_BLACK ) );
 
+  Rebuild();
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : InitWindow::~InitWindow
+// DESCRIPTION: Destroy window.
+// PARAMETERS : -
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+InitWindow::~InitWindow( void ) {
+  TLWNode *n;
+  string *str;
+
+  while ( !levels.IsEmpty() ) {
+    n = static_cast<TLWNode *>( levels.RemHead() );
+    str = (string *)n->UserData();
+    delete str;
+    delete n;
+  }
+
+  while ( !campaigns.IsEmpty() ) {
+    n = static_cast<TLWNode *>( campaigns.RemHead() );
+    str = (string *)n->UserData();
+    delete str;
+    delete n;
+  }
+
+  while ( !saves.IsEmpty() ) {
+    n = static_cast<TLWNode *>( saves.RemHead() );
+    str = (string *)n->UserData();
+    delete str;
+    delete n;
+  }
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : InitWindow::Rebuild
+// DESCRIPTION: (Re)create all window content. This is used both in the
+//              constructor and when the window size has changed.
+// PARAMETERS : -
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void InitWindow::Rebuild( void ) {
+  RemoveAllWidgets();
+
   // calculate window dimensions
   const char *typelbl = MSG(MSG_B_GAME_TYPE);
   const char *mapslbl = MSG(MSG_B_MAP_TYPE);
@@ -110,7 +157,7 @@ InitWindow::InitWindow( View *view, Window *title ) :
   difflabels[3] = 0;
 
   gtypewidget = new CycleWidget( IW_WIDGET_TYPE, xoff, 5,
-                (w - xoff - 10) / 2, wdh, WIDGET_ALIGN_LEFT,
+                (w - xoff - 10) / 2, wdh, 0,
                 typelbl, CFOptions.GetGameType(), typelabels, this );
   gtypewidget->SetHook( this );
 
@@ -143,9 +190,7 @@ InitWindow::InitWindow( View *view, Window *title ) :
   mapwidget = new MapWidget( 0, maxmap.x, maxmap.y, maxmap.w, maxmap.h,
               0, this );
   campinfowidget = new TextScrollWidget( 0, maxmap.x, maxmap.y,
-              maxmap.w, maxmap.h, 0, 0, 0, this );
-  RemoveWidget( campinfowidget );
-
+              maxmap.w, maxmap.h, 0, WIDGET_HIDDEN, 0, this );
 
   wd = new ButtonWidget( IW_WIDGET_START, 5, h - wdh - 2, (w - 10) / 2 - 2, wdh,
                 WIDGET_DEFAULT, MSG(MSG_B_START), this );
@@ -154,45 +199,20 @@ InitWindow::InitWindow( View *view, Window *title ) :
   wd = new ButtonWidget( GUI_QUIT, wd->LeftEdge() + wd->Width() + 4,
                 wd->TopEdge(), wd->Width(), wdh,
                 0, MSG(MSG_B_EXIT), this );
-
   Draw();
 }
 
 ////////////////////////////////////////////////////////////////////////
-// NAME       : InitWindow::~InitWindow
-// DESCRIPTION: Destroy window.
+// NAME       : InitWindow::VideoModeChange
+// DESCRIPTION: This method is called by the view whenever the video
+//              resolution changes. We can then resize the window and
+//              widgets accordingly.
 // PARAMETERS : -
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-InitWindow::~InitWindow( void ) {
-  TLWNode *n;
-  string *str;
-
-  while ( !levels.IsEmpty() ) {
-    n = static_cast<TLWNode *>( levels.RemHead() );
-    str = (string *)n->UserData();
-    delete str;
-    delete n;
-  }
-
-  while ( !campaigns.IsEmpty() ) {
-    n = static_cast<TLWNode *>( campaigns.RemHead() );
-    str = (string *)n->UserData();
-    delete str;
-    delete n;
-  }
-
-  while ( !saves.IsEmpty() ) {
-    n = static_cast<TLWNode *>( saves.RemHead() );
-    str = (string *)n->UserData();
-    delete str;
-    delete n;
-  }
-
-  // one of these is not attached to the window
-  if ( mtypewidget->GetValue() == 1 ) delete mapwidget;
-  else delete campinfowidget;
+void InitWindow::VideoModeChange( void ) {
+  Rebuild();
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -297,12 +317,16 @@ GUI_Status InitWindow::WidgetActivated( Widget *button, Window *win ) {
       levwidget->SwitchList( &levels, -1 );
       gtypewidget->Enable();
       diffwidget->Enable();
+      campinfowidget->Hide();
+      mapwidget->Unhide();
+      CFOptions.SetCampaign( false );
       break;
     case MAPTYPE_CAMPAIGN:
       levwidget->SwitchList( &campaigns, -1 );
       gtypewidget->Disable();
-      RemoveWidget( mapwidget );
-      AddWidget( campinfowidget );
+      diffwidget->Enable();
+      campinfowidget->Unhide();
+      mapwidget->Hide();
       CFOptions.SetCampaign( true );
       break;
     case MAPTYPE_SAVE:
@@ -310,8 +334,9 @@ GUI_Status InitWindow::WidgetActivated( Widget *button, Window *win ) {
       mapwidget->SetSize( maxmap.x, maxmap.y, maxmap.w, maxmap.h );
       levwidget->SwitchList( &saves, -1 );
       diffwidget->Disable();
-      RemoveWidget( campinfowidget );
-      AddWidget( mapwidget );
+      gtypewidget->Disable();
+      campinfowidget->Hide();
+      mapwidget->Unhide();
       CFOptions.SetCampaign( false );
       break;
     }
@@ -365,8 +390,8 @@ GUI_Status InitWindow::WidgetActivated( Widget *button, Window *win ) {
 ////////////////////////////////////////////////////////////////////////
 // NAME       : InitWindow::CompleteFilesList
 // DESCRIPTION: For the levels list display we need some information
-//              about the maps (password protection, 1 or 2 players).
-//              This information is attached to the list items.
+//              about the maps (campaign info, 1 or 2 players). This
+//              information is attached to the list items.
 // PARAMETERS : list - list of files created using create_files_list()
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
@@ -377,9 +402,9 @@ void InitWindow::CompleteFilesList( TLWList &list ) {
   for ( TLWNode *n = static_cast<TLWNode *>(list.Head()), *next; n; n = next ) {
     next = static_cast<TLWNode *>(n->Next());
     string *fullname = (string *)n->UserData();
-    bool remove = false;
+    bool campaign = false, skirmish = false, remove = false;
 
-    Mission *m = LoadMission( fullname->c_str() );
+    Mission *m = LoadMission( fullname->c_str(), false );
     if ( m ) {
       string filename;
       string title;
@@ -394,21 +419,28 @@ void InitWindow::CompleteFilesList( TLWList &list ) {
         // new mission
         // campaign maps are only available if they have been unlocked
         // by playing the campaign
-        remove = (m->GetFlags() & GI_CAMPAIGN) &&  // campaign map
-                 (m->GetCampaignName() == 0)   &&  // not first map of a campaign
-                 CFOptions.IsLocked( filename );   // not played yet
+        campaign = ( (m->GetFlags() & GI_CAMPAIGN) &&   // campaign map
+                     (m->GetCampaignName() != 0) );     // first map of a campaign
 
-        if ( !remove ) {
-          if ( m->GetCampaignName() ) {
-            TLWNode *cnode = new TLWNode( m->GetCampaignName() );
-            string *full_path = new string(*fullname);
-            cnode->SetUserData( full_path );
-            campaigns.InsertNodeSorted( cnode );
-          }
+        skirmish = ( (m->GetFlags() & GI_SKIRMISH) &&     // skirmish map
+                     (!(m->GetFlags() & GI_CAMPAIGN) ||   // no campaign map
+                      campaign ||                         // first campaign map
+                      !CFOptions.IsLocked( filename )) ); // or already played
 
+        if ( (m->GetFlags() & GI_CAMPAIGN) &&   // campaign map
+             (m->GetCampaignName() != 0) ) {    // first map of a campaign
+          TLWNode *cnode = new TLWNode( m->GetCampaignName() );
+          string *full_path = new string(*fullname);
+          cnode->SetUserData( full_path );
+          campaigns.InsertNodeSorted( cnode );
+        }
+
+        if ( skirmish ) {
           title.assign( m->GetName() );
           reorder = true;
         }
+
+        remove = !skirmish;
       }
 
       if ( !remove ) {
@@ -448,17 +480,24 @@ void InitWindow::CompleteFilesList( TLWList &list ) {
 // NAME       : InitWindow::LoadMission
 // DESCRIPTION: Load a mission into memory for inspection.
 // PARAMETERS : filename - mission filename
+//              full     - whether to completely load the mission
+//                         (Mission::Load) or only initialize the parts
+//                         for the map selection (Mission::QuickLoad).
+//                         Defaults to TRUE.
 // RETURNS    : mission if successful, NULL on error
 ////////////////////////////////////////////////////////////////////////
 
-Mission *InitWindow::LoadMission( const char *filename ) const {
+Mission *InitWindow::LoadMission( const char *filename, bool full ) const {
   Mission *m = 0;
 
   File file( filename );
   if ( file.Open( "rb" ) ) {
-
+    int rc;
     m = new Mission();
-    if ( m->Load( file ) != -1 ) {
+
+    if ( full ) rc = m->Load( file );
+    else rc = m->QuickLoad( file );
+    if ( rc != -1 ) {
       m->SetLocale( CFOptions.GetLanguage() );
     } else {
       delete m;
@@ -672,6 +711,9 @@ void GenericOptionsWindow::SetLayout( unsigned short w, unsigned short h ) {
 
   if ( w < tw + 20 ) w = tw + 20;
 
+  if ( h + 25 + lfont->Height() + sfont->Height() >= view->Height() )
+    h = view->Height() - lfont->Height() - sfont->Height() - 26;
+
   clientarea = Rect( 5, 13 + lfont->Height(), w, h );
 
   Window::SetSize( w + 10, h + lfont->Height() + sfont->Height() + 25 );
