@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -140,6 +140,56 @@ void MapWindow::MoveHex( unsigned short img, Surface *surface,
   delete bgs;
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : MapWindow::FadeInHex
+// DESCRIPTION: Fade in a hex image (a unit or terrain image).
+// PARAMETERS : img  - tile identifier for the hex image
+//              hex  - destination hex position
+//              unit - if TRUE paint a unit image, otherwise the
+//                     respective terrain
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void MapWindow::FadeInHex( unsigned short img, const Point &hex, bool unit ) {
+  unsigned long firstticks, oldticks, ticks;
+  Point pos = mview->Hex2Pixel( hex );
+  const Rect dest( pos.x, pos.y, GFX_WIDTH, GFX_HEIGHT );
+  const unsigned int speed = 4 * ANIM_SPEED_UNIT;
+  const unsigned short terrain = mview->GetMap()->HexImage( hex );
+
+  Surface tile;
+  tile.Create( GFX_WIDTH, GFX_HEIGHT, DISPLAY_BPP, SDL_HWSURFACE );
+  tile.SetAlpha( SDL_ALPHA_TRANSPARENT, SDL_SRCALPHA );
+  tile.SetColorKey( Color(CF_COLOR_WHITE) );
+  tile.DisplayFormat();
+  tile.Flood( Color(CF_COLOR_WHITE) );
+
+  Unit *blocker = NULL;
+  if ( unit ) mview->DrawUnit( img, &tile, 0, 0, tile );
+  else {
+    mview->DrawTerrain( img, &tile, 0, 0, tile );
+    blocker = mview->GetMap()->GetUnit( hex );
+  }
+
+  firstticks = oldticks = SDL_GetTicks();
+  do {
+    ticks = SDL_GetTicks() - firstticks;
+    int alpha = SDL_ALPHA_OPAQUE * ticks / speed;
+    if ( alpha > SDL_ALPHA_OPAQUE ) alpha = SDL_ALPHA_OPAQUE;
+
+    tile.SetAlpha( alpha, SDL_SRCALPHA );
+    mview->DrawTerrain( terrain, this, pos.x, pos.y, dest );
+    tile.Blit( this, tile, pos.x, pos.y );
+    if ( blocker )
+      mview->DrawUnit( blocker->Image(), this, pos.x, pos.y, dest );
+    Show( dest );
+
+    if ( ticks - oldticks < 30 ) SDL_Delay( 25 );
+    oldticks = ticks;
+  } while ( ticks < speed );
+
+  Show( dest );
+}
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : MapWindow::BoxAvoidHexes
@@ -174,6 +224,22 @@ void MapWindow::BoxAvoidHexes( Rect &box, const Point &hex1, const Point &hex2 )
   }
 } 
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : MapWindow::DisplayHex
+// DESCRIPTION: Make sure the requested hex is visible and update the
+//              display. Only call this function if the map view is
+//              enabled.
+// PARAMETERS : hex - hex to view
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void MapWindow::DisplayHex( const Point &hex ) {
+  if ( mview->FlagSet( MV_DIRTY ) || !mview->HexVisible( hex ) ) {
+    mview->CenterOnHex( hex );
+    mview->UnsetFlags( MV_DIRTY );
+    Show( *mview );
+  }
+}
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : MapWindow::MoveCursor
