@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@
 #include "misc.h"
 
 #define ANIM_SPEED_UNIT		150  // time it takes for a unit to move one hex (ms)
-#define ANIM_SPEED_CURSOR	40   // speed when moveing cursor
+#define ANIM_SPEED_CURSOR	40   // speed when moving cursor
 
 #define DEFAULT_PANEL_HEIGHT	(20)
 
@@ -70,11 +70,18 @@ public:
   void MoveHex( unsigned short img, Surface *surface, const Point &hex1,
                 const Point &hex2, unsigned short speed );
   Point MoveCursor( Direction dir );
-  void BoxAvoidHexes( Rect &rect, const Point &hex1, const Point &hex2 ) const;
+  void FadeInUnit( unsigned short img, const Point &hex )
+       { FadeInHex( img, hex, true ); }
+  void FadeInTerrain( unsigned short img, const Point &hex )
+       { FadeInHex( img, hex, false ); }
 
+  void DisplayHex( const Point &hex );
+  void BoxAvoidHexes( Rect &rect, const Point &hex1, const Point &hex2 ) const;
   void DrawUnitInfo( const UnitType *type, Surface *dest, const Rect &rect );
 
 private:
+  void FadeInHex( unsigned short img, const Point &hex, bool unit );
+
   MapView *mview;
   Panel *panel;
   Image speed_icon;
