@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -36,7 +36,6 @@
 Unit::Unit( const UnitType *type, Player *player, unsigned short id, const Point &pos ) :
       MapObject( MO_UNIT ), u_pos(pos), u_id(id), u_type(type), u_player(0) {
   u_flags = type->Flags();
-  u_moves = 0;
   u_facing = NORTH;
   u_group = MAX_GROUP_SIZE;
   u_xp = 0;
@@ -60,7 +59,6 @@ int Unit::Load( File &file, const UnitType *type, Player *player ) {
   u_pos.y = file.Read16();
   u_flags = file.Read32();
   u_id = file.Read16();
-  u_moves = file.Read8();
   u_facing = file.Read8();
   u_group = file.Read8();
   u_xp = file.Read8();
@@ -90,7 +88,6 @@ int Unit::Save( File &file ) const {
   file.Write32( u_flags );
   file.Write16( u_id );
 
-  file.Write8( u_moves );
   file.Write8( u_facing );
   file.Write8( u_group );
   file.Write8( u_xp );
@@ -159,7 +156,6 @@ void Unit::SetPosition( short x, short y ) {
 void Unit::Attack( const Unit *enemy ) {
   u_target = enemy->Position();
   SetFlags( U_ATTACKED|U_DONE );
-  u_moves = 0;
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -178,7 +174,7 @@ unsigned char Unit::DefensiveStrength( void ) const {
 // NAME       : Unit::OffensiveStrength
 // DESCRIPTION: Return the combat value of the unit against the given
 //              target.
-// PARAMETERS : target   - unit to attack
+// PARAMETERS : target - unit to attack
 // RETURNS    : offensive strength
 ////////////////////////////////////////////////////////////////////////
 
@@ -284,7 +280,6 @@ void Unit::Repair( void ) {
   if ( u_group < MAX_GROUP_SIZE ) {
     u_xp = MAX( 0, u_xp - (MAX_GROUP_SIZE - u_group)/2 );
     u_group = MAX_GROUP_SIZE;
-    u_moves = 0;
     SetFlags( U_DONE );	// can't move this turn
   }
 }
@@ -301,23 +296,26 @@ unsigned char Unit::WeaponRange( const Unit *u ) const {
   unsigned char range;
 
   if ( u->IsAircraft() ) range = u_type->MaxFOF(U_AIR);
-  else if ( u->IsShip() || u->IsFloating() )
-    range = u_type->MaxFOF(U_SHIP);
+  else if ( u->IsShip() || u->IsFloating() ) range = u_type->MaxFOF(U_SHIP);
   else range = u_type->MaxFOF(U_GROUND);
-
   return range;
 }
 
 ////////////////////////////////////////////////////////////////////////
-// NAME       : Unit::RefreshMoves
-// DESCRIPTION: Get the unit back to full movement points.
+// NAME       : Unit::Moves
+// DESCRIPTION: Get movement points for this unit.
 // PARAMETERS : -
-// RETURNS    : -
+// RETURNS    : movement points
 ////////////////////////////////////////////////////////////////////////
 
-void Unit::RefreshMoves( void ) {
-  if ( IsMine() ) {
-    if ( IsSheltered() ) u_moves = 1; // MCOST_MIN
-  } else u_moves = u_type->Speed();
+unsigned char Unit::Moves( void ) const {
+  unsigned char mp;
+
+  if ( Flags() & (U_ATTACKED|U_MOVED|U_DONE) ) mp = 0;
+  // mines inside shops or transporters get MCOST_MIN points so they
+  // can move out
+  else if ( IsMine() && IsSheltered() ) mp = 1;
+  else mp = u_type->Speed();
+  return mp;
 }
 
