@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -37,6 +37,7 @@ public:
   MapObject( unsigned short type ) : mo_type(type) {}
   virtual const char *Name( void ) const = 0;
   virtual Player *Owner( void ) const = 0;
+  virtual const Point &Position( void ) const = 0;
 
   bool IsUnit( void ) const { return mo_type == MO_UNIT; }
   bool IsShop( void ) const { return mo_type == MO_BUILDING; }
@@ -60,8 +61,9 @@ public:
   unsigned long Flags( void ) const { return u_flags; }
   unsigned char GroupSize( void ) const { return u_group; }
   unsigned short ID( void ) const { return u_id; }
-  unsigned short Image( void ) const { return BaseImage() + u_facing; }
-  unsigned char Moves( void ) const { return u_moves; }
+  unsigned char Facing( void ) const { return u_facing; }
+  unsigned short Image( void ) const { return BaseImage() + Facing(); }
+  unsigned char Moves( void ) const;
   const char *Name( void ) const { return u_type->Name(); }
   Player *Owner( void ) const { return u_player; }
   const Point &Position( void ) const { return u_pos; }
@@ -74,8 +76,7 @@ public:
   void AwardXP( unsigned char xp );
   void SetOwner( Player *player );
   virtual void SetPosition( short x, short y );
-  void SetMoves( unsigned char moves ) { u_moves = moves; }
-  void RefreshMoves( void );
+  void SetGroupSize( unsigned char size ) { u_group = size; }
 
   void SetFlags( unsigned long f ) { u_flags |= (f); }
   void UnsetFlags( unsigned long f ) { u_flags &= (~f); }
@@ -86,7 +87,6 @@ public:
   bool IsMine( void ) const { return (u_flags & U_MINE) != 0; }
   bool IsTransport( void ) const { return (u_flags & U_TRANSPORT) != 0; }
   bool IsMedic( void ) const { return (u_flags & U_MEDIC) != 0; }
-  bool IsPioneer( void ) const { return (u_flags & U_PIONEERS) != 0; }
   bool IsMinesweeper( void ) const { return (u_flags & U_MINESWEEPER) != 0; }
 
   bool IsSlow( void ) const { return (u_flags & U_SLOW) != 0; }
@@ -122,7 +122,6 @@ protected:
   unsigned long u_flags;
   unsigned short u_id;
   
-  unsigned char u_moves;
   unsigned char u_facing;	// direction
   unsigned char u_group;	// group size
   unsigned char u_xp;		// experience
