@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -344,26 +344,26 @@ void ContainerWindow::Draw( void ) {
   Window::Draw();
   Gam->GetMapWindow()->DrawUnitInfo( NULL, this, unitinfo );
 
-  // draw a separator bar
-  unsigned short sep = unitinfo.x;
+  unsigned short xoff;
+  Rect boxrect( unitinfo.x, 10, w - unitinfo.x - 10, lfont->Height() + 20 );
 
   // show the name of the unit/building
-  DrawBox( Rect( sep, 10, w - sep - 10, lfont->Height() + 20 ), BOX_RAISED );
+  DrawBox( boxrect, BOX_RAISED );
 
-  string namestr( dynamic_cast<MapObject *>(c)->Name() );
-  unsigned short nlen = lfont->FitText( namestr.c_str(), w - sep - 14, false );
-  if ( nlen < namestr.size() ) {
-    namestr.erase( nlen - 1 );
-    namestr += "...";
-  }
+  boxrect.x += 5;
+  boxrect.w -= 10;
 
-  const char *name = namestr.c_str();
-  lfont->Write( name, this,
-                sep + 3 + (w - sep - 10 - lfont->TextWidth(name)) / 2, 23,
-                view->GetBGPen() );
-  lfont->Write( name, this,
-                sep + (w - sep - 10 - lfont->TextWidth(name)) / 2, 20,
-                view->GetFGPen() );
+  const char *name = dynamic_cast<MapObject *>(c)->Name();
+  unsigned short nlen = lfont->FitText( name, boxrect.w, false );
+  if ( nlen < strlen(name) ) xoff = 0;
+  else xoff = (boxrect.w - lfont->TextWidth( name )) / 2;
+
+  Color col = lfont->GetColor();
+  lfont->SetColor( view->GetBGPen() );
+  lfont->WriteEllipsis( name, this, boxrect.x + xoff + 3, 23, boxrect );
+  lfont->SetColor( view->GetFGPen() );
+  lfont->WriteEllipsis( name, this, boxrect.x + xoff, 20, boxrect );
+  lfont->SetColor( col );
 
   // show crystal storage
   crystals_icon.Draw( this, w - 5 - ICON_WIDTH, lfont->Height() + 30 );
