@@ -1,5 +1,5 @@
 // CoMET - The Crimson Fields Map Editing Tool
-// Copyright (C) 2002-2003 Jens Granseuer
+// Copyright (C) 2002-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,8 +20,6 @@
 // building.cpp
 ////////////////////////////////////////////////////////////////////////
 
-#include "SDL_endian.h"
-
 #include "building.h"
 
 ////////////////////////////////////////////////////////////////////////
@@ -30,13 +28,12 @@
 // PARAMETERS : pos  - position on map
 //              id   - building indentifier
 //              pid  - identifier of controlling player
-//              name - building name
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
 Building::Building( const Point &pos, unsigned short id,
-                    unsigned char pid, const char *name ) :
-          MapObject( MO_BUILDING, name ) {
+                    unsigned char pid ) :
+          MapObject( MO_BUILDING ) {
   b_id = id;
   b_pos = pos;
   b_flags = BLD_WORKSHOP;
@@ -47,7 +44,8 @@ Building::Building( const Point &pos, unsigned short id,
   b_pid = pid;
   b_minweight = 0;
   b_maxweight = 99;
-  strcpy( b_name, name );
+  b_name_id = -1;
+  b_name = 0;
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -57,22 +55,21 @@ Building::Building( const Point &pos, unsigned short id,
 // RETURNS    : 0 on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int Building::Load( SDL_RWops *file ) {
-  b_id = SDL_ReadLE16( file );
-  b_pos.x = SDL_ReadLE16( file );
-  b_pos.y = SDL_ReadLE16( file );
-  b_flags = SDL_ReadLE16( file );
-  b_crystals = SDL_ReadLE16( file );
-  b_maxcrystals = SDL_ReadLE16( file );
-  b_uprod = SDL_ReadLE32( file );
+int Building::Load( File &file ) {
+  b_id = file.Read16();
+  b_pos.x = file.Read16();
+  b_pos.y = file.Read16();
+  b_flags = file.Read16();
+  b_crystals = file.Read16();
+  b_maxcrystals = file.Read16();
+  b_uprod = file.Read32();
 
-  SDL_RWread( file, &b_cprod, 1, 1 );
-  SDL_RWread( file, &b_pid, 1, 1 );
-  SDL_RWread( file, &b_minweight, 1, 1 );
-  SDL_RWread( file, &b_maxweight, 1, 1 );
-  SDL_RWread( file, b_name, 1, 20 );
-
-  SetName( b_name );
+  b_cprod = file.Read8();
+  b_pid = file.Read8();
+  b_minweight = file.Read8();
+  b_maxweight = file.Read8();
+  b_name_id = file.Read8();
+  b_name = 0;
   return 0;
 }
 
@@ -83,20 +80,20 @@ int Building::Load( SDL_RWops *file ) {
 // RETURNS    : 0 on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int Building::Save( SDL_RWops *file ) const {
-  SDL_WriteLE16( file, b_id );
-  SDL_WriteLE16( file, b_pos.x );
-  SDL_WriteLE16( file, b_pos.y );
-  SDL_WriteLE16( file, b_flags );
-  SDL_WriteLE16( file, b_crystals );
-  SDL_WriteLE16( file, b_maxcrystals );
-  SDL_WriteLE32( file, b_uprod );
+int Building::Save( File &file ) const {
+  file.Write16( b_id );
+  file.Write16( b_pos.x );
+  file.Write16( b_pos.y );
+  file.Write16( b_flags );
+  file.Write16( b_crystals );
+  file.Write16( b_maxcrystals );
+  file.Write32( b_uprod );
 
-  SDL_RWwrite( file, &b_cprod, 1, 1 );
-  SDL_RWwrite( file, &b_pid, 1, 1 );
-  SDL_RWwrite( file, &b_minweight, 1, 1 );
-  SDL_RWwrite( file, &b_maxweight, 1, 1 );
-  SDL_RWwrite( file, b_name, 1, 20 );
+  file.Write8( b_cprod );
+  file.Write8( b_pid );
+  file.Write8( b_minweight );
+  file.Write8( b_maxweight );
+  file.Write8( b_name_id );
   return 0;
 }
 
@@ -109,11 +106,10 @@ int Building::Save( SDL_RWops *file ) const {
 
 int Building::Export( ofstream &file, const UnitSet *uset ) const {
   file << "[building]\n";
-  file << "name = " << b_name << '\n';
+  file << "name = " << (short)b_name_id << '\n';
   file << "player = " << (b_pid+1)%3 << '\n';
   file << "id = " << b_id << '\n';
-  file << "xpos = " << b_pos.x << '\n';
-  file << "ypos = " << b_pos.y << '\n';
+  file << "pos = " << b_pos.x << '/' << b_pos.y << '\n';
 
   if ( (b_flags & BLD_WORKSHOP) != 0 ) file << "type = workshop\n";
   if ( (b_flags & BLD_FACTORY) != 0 ) {
@@ -135,17 +131,3 @@ int Building::Export( ofstream &file, const UnitSet *uset ) const {
   return 0;
 }
 
-////////////////////////////////////////////////////////////////////////
-// NAME       : Building::SetName
-// DESCRIPTION: Set the building name.
-// PARAMETERS : name - building descriptor
-// RETURNS    : -
-////////////////////////////////////////////////////////////////////////
-
-void Building::SetName( const char *name ) {
-  if ( name ) {
-    strncpy( b_name, name, 19 );
-    b_name[19] = '\0';
-  } else b_name[0] = '\0';
-}
-
