@@ -1,5 +1,5 @@
 // CoMET - The Crimson Fields Map Editing Tool
-// Copyright (C) 2002-2004 Jens Granseuer
+// Copyright (C) 2002-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 #include "filewindow.h"
 #include "gfxwidget.h"
 #include "fileio.h"
+#include "strutil.h"
 #include "main.h"
 
 // button IDs for the context menu
@@ -46,13 +47,14 @@ const short EdWindow::B_ID_BLD_CREATE = B_ID_MENU_BASE + 5;
 const short EdWindow::B_ID_BLD_EDIT   = B_ID_MENU_BASE + 6;
 const short EdWindow::B_ID_BLD_DEL    = B_ID_MENU_BASE + 7;
 const short EdWindow::B_ID_EVENTS     = B_ID_MENU_BASE + 8;
-const short EdWindow::B_ID_NEW        = B_ID_MENU_BASE + 9;
-const short EdWindow::B_ID_LOAD       = B_ID_MENU_BASE + 10;
-const short EdWindow::B_ID_SAVE       = B_ID_MENU_BASE + 11;
-const short EdWindow::B_ID_EXPORT     = B_ID_MENU_BASE + 12;
-const short EdWindow::B_ID_VALIDATE   = B_ID_MENU_BASE + 13;
-const short EdWindow::B_ID_SETTINGS   = B_ID_MENU_BASE + 14;
-const short EdWindow::B_ID_QUIT       = B_ID_MENU_BASE + 15;
+const short EdWindow::B_ID_MESSAGES   = B_ID_MENU_BASE + 9;
+const short EdWindow::B_ID_NEW        = B_ID_MENU_BASE + 10;
+const short EdWindow::B_ID_LOAD       = B_ID_MENU_BASE + 11;
+const short EdWindow::B_ID_SAVE       = B_ID_MENU_BASE + 12;
+const short EdWindow::B_ID_EXPORT     = B_ID_MENU_BASE + 13;
+const short EdWindow::B_ID_VALIDATE   = B_ID_MENU_BASE + 14;
+const short EdWindow::B_ID_SETTINGS   = B_ID_MENU_BASE + 15;
+const short EdWindow::B_ID_QUIT       = B_ID_MENU_BASE + 16;
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : EdWindow::EdWindow
@@ -143,6 +145,13 @@ void EdWindow::PrintCursorPos( void ) {
   if ( mission ) {
     sprintf( posbuf, "X/Y: %d/%d", mv.Cursor().x, mv.Cursor().y );
     sfont->Write( posbuf, this, coord.x, coord.y );
+
+    MapObject *obj = mission->GetMap().GetMapObject( mv.Cursor() );
+    if ( obj && obj->Name() ) {
+      string name = StringUtil::strprintf( "(%s)", obj->Name() );
+      sfont->WriteEllipsis( name.c_str(), this,
+             coord.x + sfont->TextWidth( posbuf ) + 5, coord.y, coord );
+    }
   }
 }
 
@@ -154,66 +163,66 @@ void EdWindow::PrintCursorPos( void ) {
 ////////////////////////////////////////////////////////////////////////
 
 GUI_Status EdWindow::HandleEvent( const SDL_Event &event ) {
-  GUI_Status rc = Window::HandleEvent( event );
 
+  if ( !mission ) {
+    if ( event.type == SDL_KEYDOWN )
+      ShowContextMenu( Point(-1, -1) );
+    else if ( event.type == SDL_MOUSEBUTTONDOWN )
+      ShowContextMenu( Point(event.button.x - x, event.button.y - y) );
+
+    return GUI_OK;
+  }
+
+  GUI_Status rc = Window::HandleEvent( event );
   if ( rc == GUI_OK ) {
 
     // check for keyboard commands
     if ( event.type == SDL_KEYDOWN ) {
 
-      if ( !mission )
-        ShowContextMenu( Point(-1, -1) );
-      else {
+      switch ( event.key.keysym.sym ) {
+      case SDLK_KP1: case SDLK_KP2: case SDLK_KP3:
+      case SDLK_KP4: case SDLK_KP6: case SDLK_KP7:
+      case SDLK_KP8: case SDLK_KP9:
+      case SDLK_LEFT: case SDLK_RIGHT: case SDLK_UP: case SDLK_DOWN:
+        MoveCursor( event.key.keysym.sym );
+        break;
+      case SDLK_SPACE:      // equivalent to left mouse button
+        LeftMouseButton( mv.Cursor() );
+        break;
+      case SDLK_q:
+        Quit();
+        break;
+      case SDLK_ESCAPE:
+        tile_wd->Select( -1 );
+        unit_wd->Select( -1 );
+        break;
+      case SDLK_RETURN:
+        ShowContextMenu( mv.Cursor() );
 
-        switch ( event.key.keysym.sym ) {
-        case SDLK_KP1: case SDLK_KP2: case SDLK_KP3:
-        case SDLK_KP4: case SDLK_KP6: case SDLK_KP7:
-        case SDLK_KP8: case SDLK_KP9:
-        case SDLK_LEFT: case SDLK_RIGHT: case SDLK_UP: case SDLK_DOWN:
-          MoveCursor( event.key.keysym.sym );
-          break;
-        case SDLK_SPACE:      // equivalent to left mouse button
-          LeftMouseButton( mv.Cursor() );
-          break;
-        case SDLK_q:
-          Quit();
-          break;
-        case SDLK_ESCAPE:
-          tile_wd->Select( -1 );
-          unit_wd->Select( -1 );
-          break;
-        case SDLK_RETURN:
-          ShowContextMenu( mv.Cursor() );
-
-        case SDLK_u: {		// edit unit
-          Unit *u = mv.GetMap()->GetUnit( mv.Cursor() );
-          if ( u ) {
-            EdUnitWindow *euw = new EdUnitWindow( *u, *mission, view );
-            euw->SetMapView( mv );
-          }
-          break; }
-        case SDLK_b: {		// edit building
-          Building *b = mv.GetMap()->GetBuilding( mv.Cursor() );
-          if ( b ) new EdBuildingWindow( *b, *mission, view );
-          break; }
-
-        default:
-          break;
+      case SDLK_u: {		// edit unit
+        Unit *u = mv.GetMap()->GetUnit( mv.Cursor() );
+        if ( u ) {
+          EdUnitWindow *euw = new EdUnitWindow( *u, *mission, view );
+          euw->SetMapView( mv );
         }
+        break; }
+      case SDLK_b: {		// edit building
+        Building *b = mv.GetMap()->GetBuilding( mv.Cursor() );
+        if ( b ) new EdBuildingWindow( *b, *mission, view );
+        break; }
+
+      default:
+        break;
       }
 
     } else if ( event.type == SDL_MOUSEBUTTONDOWN ) {
-      if ( !mission )
-        ShowContextMenu( Point(event.button.x - x, event.button.y - y) );
-      else {
-        Point pos;
-        bool validhex = !mv.Pixel2Hex( event.button.x - x, event.button.y - y, pos );
+      Point pos;
+      bool validhex = !mv.Pixel2Hex( event.button.x - x, event.button.y - y, pos );
 
-        if ( event.button.button == SDL_BUTTON_LEFT ) {
-          if ( validhex ) LeftMouseButton( pos );
-        } else if ( event.button.button == SDL_BUTTON_RIGHT ) {
-           ShowContextMenu( validhex ? pos : Point(-1,-1) );
-        }
+      if ( event.button.button == SDL_BUTTON_LEFT ) {
+        if ( validhex ) LeftMouseButton( pos );
+      } else if ( event.button.button == SDL_BUTTON_RIGHT ) {
+         ShowContextMenu( validhex ? pos : Point(-1,-1) );
       }
     }
   }
@@ -285,7 +294,8 @@ void EdWindow::LeftMouseButton( const Point &hex ) {
   Map *map = mv.GetMap();
 
   if ( mode == ED_MODE_TERRAIN ) {
-    if ( map->IsBuilding( hex ) && !(selected & TT_ENTRANCE) && map->GetBuilding( hex ) )
+    if ( map->IsBuilding( hex ) && map->GetBuilding(hex) &&
+       !(map->GetTerrainSet()->GetTerrainInfo(selected)->tt_type & TT_ENTRANCE) )
       new NoteWindow( "Warning", "You have just removed a shop entrance. "
                       "The corresponding shop has NOT been deleted! "
                       "(to do so restore the entrance and delete the shop before "
@@ -314,47 +324,26 @@ void EdWindow::LeftMouseButton( const Point &hex ) {
 
 Mission *EdWindow::LoadMission( const char *file, bool filereq ) {
   string fn;
+  Mission *ms = NULL;
 
   if ( filereq )
     fn = GetFileName( file, ".lev", mapdir, WIN_FILE_LOAD );
   else fn = file;
 
   if ( fn.length() > 0 ) {
-    delete mission;
-    mv.Disable();
-
-    mission = new Mission();
-    if ( mission ) {
-      int err = mission->Load( fn.c_str() );
+    ms = new Mission();
+    if ( ms ) {
+      int err = ms->Load( fn.c_str() );
       if ( err ) {
-        delete mission;
-        mission = NULL;
+        delete ms;
         new NoteWindow( "Error", "Couldn't load map. This could be a file type mismatch.", 0, view );
       } else {
-        Map &map = mission->GetMap();
-        mode = ED_MODE_VIEW;
-        mv.SetMap( &map );
-        mv.SetCursor( Point(0,0) );
-
-        tile_wd->SetTerrainSet( map.GetTerrainSet(), GFX_WIDTH, GFX_HEIGHT );
-        unit_wd->SetUnitSet( map.GetUnitSet(), GFX_WIDTH, GFX_HEIGHT );
-
-        short max = MAX( map.Height() * GFX_HEIGHT + GFX_OVERLAP_Y - mv.Height(), 0 );
-        ud_wd->Adjust( 0, max, mv.Height() );
-        ud_wd->ScrollTo( 0 );
-
-        max = MAX( map.Width() * (GFX_WIDTH - GFX_OVERLAP_X) + GFX_OVERLAP_X - mv.Width(), 0 );
-        lr_wd->Adjust( 0, max, mv.Width() );
-        lr_wd->ScrollTo( 0 );
-
-        mv.Enable();
-        Draw();
-        Show();
+        SetNewMission( ms );
       }
     }
   }
 
-  return mission;
+  return ms;
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -401,65 +390,6 @@ GUI_Status EdWindow::ExportMission( const char *file, bool filereq ) {
   return rc;
 }
 
-////////////////////////////////////////////////////////////////////////
-// NAME       : EdWindow::NewMission
-// DESCRIPTION: Create a new mission.
-// PARAMETERS : size - map size
-//              tset - tile set name
-//              uset - unit set name
-// RETURNS    : GUI status
-////////////////////////////////////////////////////////////////////////
-
-GUI_Status EdWindow::NewMission( const Point &size, const string &tset,
-                                 const string &uset ) {
-  TerrainSet *ts = new TerrainSet;
-  UnitSet *us = new UnitSet;
-
-  string tname( get_data_dir() + tset + ".tiles" );
-  File tfile( tname );
-
-  string uname( get_data_dir() + uset + ".units" );
-  File ufile( uname );
-
-  if ( !tfile.Open("rb") || ts->Load( tfile, tset.c_str() ) ) {
-    new NoteWindow( "Error", "Tile set not available", 0, view );
-    delete ts;
-  } else if ( !ufile.Open("rb") || us->Load( ufile, uset.c_str() ) ) {
-    new NoteWindow( "Error", "Unit set not available", 0, view );
-    delete ts;
-    delete us;
-  } else {
-    delete mission;
-
-    mv.Disable();
-    mode = ED_MODE_VIEW;
-
-    mission = new Mission( size, ts, us );
-    Map &map = mission->GetMap();
-    mv.SetMap( &map );
-    mv.SetCursor( Point(0,0) );
-
-    tile_wd->SetTerrainSet( ts, GFX_WIDTH, GFX_HEIGHT );
-    unit_wd->SetUnitSet( us, GFX_WIDTH, GFX_HEIGHT );
-
-    short max = MAX( map.Height() * GFX_HEIGHT + GFX_OVERLAP_Y - mv.Height(), 0 );
-    ud_wd->Adjust( 0, max, mv.Height() );
-    ud_wd->ScrollTo( 0 );
-
-    max = MAX( map.Width() * (GFX_WIDTH - GFX_OVERLAP_X) + GFX_OVERLAP_X - mv.Width(), 0 );
-    lr_wd->Adjust( 0, max, mv.Width() );
-    lr_wd->ScrollTo( 0 );
-
-    mv.Enable();
-    Draw();
-    Show();
-  }
-
-  tfile.Close();
-  ufile.Close();
-  return GUI_OK;
-}
-
 ////////////////////////////////////////////////////////////////////////
 // NAME       : EdWindow::GetFileName
 // DESCRIPTION: Get the name of a file to load or to save to. Pop up a
@@ -628,6 +558,8 @@ GUI_Status EdWindow::WidgetActivated( Widget *widget, Window *win ) {
     case B_ID_UNIT_DEL:
       mission->DeleteUnit( map->GetUnit(selected_hex) );
       Show( mv.UpdateHex( selected_hex ) );
+      PrintCursorPos();
+      Show( panel );
       break;
 
     case B_ID_BLD_CREATE:
@@ -638,16 +570,22 @@ GUI_Status EdWindow::WidgetActivated( Widget *widget, Window *win ) {
       break;
     case B_ID_BLD_DEL:
       mission->DeleteBuilding( map->GetBuilding( selected_hex ) );
+      PrintCursorPos();
+      Show( panel );
       break;
 
     case B_ID_EVENTS:
       new EdEventsWindow( *mission, view );
       break;
 
+    case B_ID_MESSAGES:
+      new EdMsgWindow( mission->GetMessages(), view );
+      break;
+
     case B_ID_NMW_CREATE: {
       NewMissionWindow *nmw = static_cast<NewMissionWindow *>(win);
-      rc = NewMission( nmw->GetMapSize(), nmw->GetTileSet(),
-                       nmw->GetUnitSet() );
+      if ( nmw->GetMission() )
+        SetNewMission( nmw->GetMission() );
       break; }
     }
 
@@ -712,6 +650,7 @@ void EdWindow::ShowContextMenu( const Point &clicked ) {
     }
 
     menu->AddItem( 0, B_ID_EVENTS, 0, "_Events..." );
+    menu->AddItem( 0, B_ID_MESSAGES, 0, "_Messages..." );
     menu->AddBar( 0 );
   }
 
@@ -773,3 +712,37 @@ void EdWindow::ValidateMission( void ) const {
   }
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : EdWindow::SetNewMission
+// DESCRIPTION: Change the current mission.
+// PARAMETERS : ms - new mission
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void EdWindow::SetNewMission( Mission *ms ) {
+  delete mission;
+
+  mv.Disable();
+  mode = ED_MODE_VIEW;
+
+  mission = ms;
+  Map &map = mission->GetMap();
+  mv.SetMap( &map );
+  mv.SetCursor( Point(0,0) );
+
+  tile_wd->SetTerrainSet( map.GetTerrainSet(), GFX_WIDTH, GFX_HEIGHT );
+  unit_wd->SetUnitSet( map.GetUnitSet(), GFX_WIDTH, GFX_HEIGHT );
+
+  short max = MAX( map.Height() * GFX_HEIGHT + GFX_OVERLAP_Y - mv.Height(), 0 );
+  ud_wd->Adjust( 0, max, mv.Height() );
+  ud_wd->ScrollTo( 0 );
+
+  max = MAX( map.Width() * (GFX_WIDTH - GFX_OVERLAP_X) + GFX_OVERLAP_X - mv.Width(), 0 );
+  lr_wd->Adjust( 0, max, mv.Width() );
+  lr_wd->ScrollTo( 0 );
+
+  mv.Enable();
+  Draw();
+  Show();
+}
+
