@@ -1,5 +1,5 @@
 // CoMET - The Crimson Fields Map Editing Tool
-// Copyright (C) 2002-2004 Jens Granseuer
+// Copyright (C) 2002-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -49,8 +49,6 @@ public:
 
   Mission *LoadMission( const char *file, bool filereq );
   GUI_Status SaveMission( const char *file, bool filereq );
-  GUI_Status NewMission( const Point &size, const string &tset,
-                         const string &uset );
   GUI_Status ExportMission( const char *file, bool filereq );
   string GetFileName( const char *filename, const char *suffix,
                       const string &dirname, int flag ) const;
@@ -63,6 +61,7 @@ private:
 
   void SwapTiles( unsigned short t1, unsigned short t2 );
   void ValidateMission( void ) const;
+  void SetNewMission( Mission *ms );
 
   static const short L_ID_UNITS;
   static const short L_ID_TILES;
@@ -78,6 +77,7 @@ private:
   static const short B_ID_BLD_EDIT;
   static const short B_ID_BLD_DEL;
   static const short B_ID_EVENTS;
+  static const short B_ID_MESSAGES;
   static const short B_ID_NEW;
   static const short B_ID_LOAD;
   static const short B_ID_SAVE;
