@@ -1,5 +1,5 @@
 // CoMET - The Crimson Fields Map Editing Tool
-// Copyright (C) 2002-2004 Jens Granseuer
+// Copyright (C) 2002-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -40,15 +40,19 @@ private:
   static const short B_ID_MSG;
   static const short B_ID_PLAYER;
   static const short B_ID_DEPEND;
+  static const short B_ID_DISCARD;
   static const short B_ID_TRIGGER;
   static const short B_ID_EVENT;
   static const short B_ID_TITLE_OK;
   static const short B_ID_MSG_OK;
   static const short B_ID_DEPEND_OK;
+  static const short B_ID_DISCARD_OK;
+  static const short B_ID_DISABLED;
 
   TextScrollWidget *e_msg;
   StringWidget *e_title;
   StringWidget *e_depends;
+  StringWidget *e_discards;
   TLWList tlist;
 
   Event &event;
@@ -66,6 +70,9 @@ private:
   static const short B_ID_UNIT_OK;
   static const short N_ID_XPOS;
   static const short N_ID_YPOS;
+  static const short B_ID_DIRECTION;
+  static const short B_ID_SIZE;
+  static const short B_ID_XP;
 
   StringWidget *e_unit;
   TLWList tlist;
@@ -93,6 +100,20 @@ private:
   Mission &mission;
 };
 
+class EdEventMessageWindow : public Window, public WidgetHook {
+public:
+  EdEventMessageWindow( Event &event, Mission &mission, View *view );
+
+private:
+  GUI_Status WidgetActivated( Widget *button, Window *win );
+
+  NumberWidget *e_xpos;
+  NumberWidget *e_ypos;
+
+  Event &event;
+  Mission &mission;
+};
+
 class EdEventMiningWindow : public Window, public WidgetHook {
 public:
   EdEventMiningWindow( Event &event, Mission &mission, View *view );
@@ -115,17 +136,21 @@ private:
   Mission &mission;
 };
 
-class EdEventNextMapWindow : public Window, public WidgetHook {
+class EdEventConfigureWindow : public Window, public WidgetHook {
 public:
-  EdEventNextMapWindow( Event &event, Mission &mission, View *view );
+  EdEventConfigureWindow( Event &event, Mission &mission, View *view );
 
 private:
   GUI_Status WidgetActivated( Widget *button, Window *win );
+  const char *GetValueString( void ) const;
 
-  static const short B_ID_MAP;
-  static const short B_ID_MAP_OK;
+  static const short B_ID_SETTING;
+  static const short B_ID_MSG;
+  static const short B_ID_MSG_OK;
+  static const short S_ID_VALUE;
 
-  StringWidget *e_map;
+  ButtonWidget *e_msg_but;
+  StringWidget *e_msg;
   TLWList tlist;
 
   Event &event;
@@ -143,6 +168,7 @@ private:
   static const short B_ID_UNIT_OK;
   static const short B_ID_SHOP;
   static const short B_ID_SHOP_OK;
+  static const short B_ID_ACTION;
 
   StringWidget *e_unit;
   StringWidget *e_shop;
@@ -173,6 +199,44 @@ private:
   StringWidget *other_title;
 };
 
+class EdEventSetHexWindow : public Window, public WidgetHook {
+public:
+  EdEventSetHexWindow( Event &event, Mission &mission, View *view );
+
+private:
+  void Draw( void );
+  GUI_Status WidgetActivated( Widget *button, Window *win );
+  void DrawTerrain( unsigned short terrain, bool update );
+
+  static const short B_ID_TILE_PREV;
+  static const short B_ID_TILE_NEXT;
+  static const short N_ID_XPOS;
+  static const short N_ID_YPOS;
+
+  Rect tilepos;
+  Event &event;
+  Mission &mission;
+};
+
+class EdEventSetTimerWindow : public Window, public WidgetHook {
+public:
+  EdEventSetTimerWindow( Event &event, Mission &mission, View *view );
+
+private:
+  GUI_Status WidgetActivated( Widget *button, Window *win );
+
+  static const short B_ID_EVENT;
+  static const short B_ID_EVENT_OK;
+  static const short B_ID_OFFSET;
+  static const short N_ID_TIME;
+
+  StringWidget *e_event;
+  TLWList tlist;
+
+  Event &event;
+  Mission &mission;
+};
+
 class EdTrigHaveShopWindow : public Window, public WidgetHook {
 public:
   EdTrigHaveShopWindow( Event &event, Mission &mission, View *view );
@@ -183,7 +247,7 @@ private:
   static const short B_ID_SHOP;
   static const short B_ID_SHOP_OK;
   static const short B_ID_PLAYER;
-  static const short N_ID_TURN;
+  static const short N_ID_TIMER;
 
   StringWidget *t_shop;
   TLWList tlist;
@@ -202,7 +266,7 @@ private:
   static const short B_ID_UNIT;
   static const short B_ID_UNIT_OK;
   static const short B_ID_PLAYER;
-  static const short N_ID_TURN;
+  static const short N_ID_TIMER;
 
   StringWidget *t_unit;
   TLWList tlist;
@@ -211,9 +275,9 @@ private:
   Mission &mission;
 };
 
-class EdTrigTurnWindow : public Window, public WidgetHook {
+class EdTrigTimerWindow : public Window, public WidgetHook {
 public:
-  EdTrigTurnWindow( Event &event, View *view );
+  EdTrigTimerWindow( Event &event, View *view );
 
 private:
   GUI_Status WidgetActivated( Widget *button, Window *win );
@@ -251,8 +315,11 @@ private:
   static const short B_ID_UNIT_OK;
   static const short N_ID_POSX;
   static const short N_ID_POSY;
+  static const short B_ID_PLAYER;
+  static const short B_ID_USELECT;
 
   StringWidget *t_unit;
+  CycleWidget *t_uselect;
   TLWList tlist;
 
   Event &event;
