@@ -1,5 +1,5 @@
 // CoMET - The Crimson Fields Map Editing Tool
-// Copyright (C) 2002-2004 Jens Granseuer
+// Copyright (C) 2002-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include "filewindow.h"
 #include "fileio.h"
 #include "uiaux.h"
+#include "mapgen.h"
 
 // initialize static class variables
 const short ListExchangeWindow::B_ID_OK = 1;
@@ -38,6 +39,7 @@ const short NewMissionWindow::B_ID_UNITSET = 1;
 const short NewMissionWindow::B_ID_TILESET = 2;
 const short NewMissionWindow::B_ID_UNITSET_OK = 3;
 const short NewMissionWindow::B_ID_TILESET_OK = 4;
+const short NewMissionWindow::B_ID_RANDOM = 5;
 
 const short EdMissionSetupWindow::B_ID_NAME = 0;
 const short EdMissionSetupWindow::B_ID_INFO = 1;
@@ -53,6 +55,7 @@ const short EdMissionSetupWindow::S_ID_SEQUEL = 10;
 const short EdMissionSetupWindow::B_ID_PLAYERS = 11;
 const short EdMissionSetupWindow::S_ID_MUSIC = 12;
 const short EdMissionSetupWindow::B_ID_CAMPAIGN = 13;
+const short EdMissionSetupWindow::B_ID_SKIRMISH = 14;
 
 const short EdBuildingWindow::B_ID_PLAYER = 1;
 const short EdBuildingWindow::B_ID_WORKSHOP = 2;
@@ -89,12 +92,22 @@ const short SelectEventWindow::B_ID_CANCEL = 3;
 
 const short SelectUnitWindow::B_ID_CANCEL = 1;
 
+const short EdMsgWindow::B_ID_LANG_NEW = 1;
+const short EdMsgWindow::B_ID_LANG_DELETE = 2;
+const short EdMsgWindow::B_ID_MSG_ADD = 3;
+const short EdMsgWindow::B_ID_MSG_MODIFY = 4;
+const short EdMsgWindow::L_ID_LANGS = 5;
+const short EdMsgWindow::L_ID_MSGS = 6;
+
 // static vars
 const char *player2_labels[] = { "One", "Two", 0 };
 const char *player3_labels[] = { "One", "Two", "Neutral", 0 };
 
-static const char *dir_labels[] = { "North", "Northeast", "Southeast",
-                                    "South", "Southwest", "Northwest", 0 };
+const char *dir_labels[] = { "North", "Northeast", "Southeast",
+                             "South", "Southwest", "Northwest", 0 };
+const char *xp_labels[] = { "0", "1", "2", "3", "4", "5", "6", 0 };
+const char *size_labels[] = { "1", "2", "3", "4", "5", "6", 0 };
+
 extern const char *event_labels[];
 extern const char *etrigger_labels[];
 
@@ -299,19 +312,20 @@ GUI_Status ListSelectWindow::WidgetActivated( Widget *widget, Window *win ) {
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-NewMissionWindow::NewMissionWindow( View *view ) : Window(WIN_CENTER, view) {
-
+NewMissionWindow::NewMissionWindow( View *view ) :
+   Window(WIN_CENTER, view), mission(0), hook(0) {
   // calculate window dimensions
   Font *f = SmallFont();
-  unsigned short width = MIN( f->Width() * 38 + 58, view->Width() );
-  unsigned short height = f->Height() * 3 + 40;
-  unsigned short xoff = f->Width() * 11 + 14, yoff = 5;
+  unsigned short width = MIN( f->Width() * 38 + 20, view->Width() );
+  unsigned short height = f->Height() * 6 + 60;
+  unsigned short xoff, yoff = 5;
+  xoff = MAX(f->TextWidth("Map Width:"), f->TextWidth("Map Height:")) + 10;
 
   SetSize( width, height );
 
   m_width = new NumberWidget( 1, xoff, yoff, f->Width() * 4 + 10,
-            f->Height() + 6, 40, 20, 250, WIDGET_ALIGN_LEFT,
-            "Map _Width :", this );
+            f->Height() + 6, 40, 20, 180, WIDGET_ALIGN_LEFT,
+            "Map _Width:", this );
 
   width = MIN( f->Width() * 11 + 10, w - xoff - m_width->w - 24 - f->Width() * 12 );
   tileset = new StringWidget( 3, xoff + m_width->w + 10 + f->Width() * 9,
@@ -327,7 +341,7 @@ NewMissionWindow::NewMissionWindow( View *view ) : Window(WIN_CENTER, view) {
   yoff += m_width->Height() + 5;
 
   m_height = new NumberWidget( 2, xoff, yoff, m_width->w, m_width->h,
-            40, 20, 250, WIDGET_ALIGN_LEFT, "Map _Height:", this );
+            40, 20, 180, WIDGET_ALIGN_LEFT, "Map _Height:", this );
 
   unitset = new StringWidget( 4, tileset->x, yoff, tileset->w, tileset->h,
             "default", 10, WIDGET_ALIGN_LEFT|WIDGET_STR_CONST,
@@ -338,12 +352,32 @@ NewMissionWindow::NewMissionWindow( View *view ) : Window(WIN_CENTER, view) {
   btn->SetKey( 'u' );
   btn->SetHook( this );
 
+  yoff += btn->Height() + 5;
+  gen_random = new CheckboxWidget( B_ID_RANDOM, 5, yoff,
+               DEFAULT_CBW_SIZE, DEFAULT_CBW_SIZE, true,
+               WIDGET_STYLE_GFX|WIDGET_STYLE_NOBORDER|WIDGET_ALIGN_RIGHT,
+               "_Generate random terrain", this );
+  gen_random->SetHook( this );
+
+  yoff += gen_random->Height() + 5;
+  xoff = gen_random->LeftEdge() + 10 + f->TextWidth("Water Level");
+  gen_water = new SliderWidget( 0, xoff, yoff,
+              w - xoff - 5, DEFAULT_SLIDER_SIZE, 0, 10, 4, 1,
+              WIDGET_HSCROLL|WIDGET_ALIGN_LEFT, "W_ater Level", this );
+
+  yoff += gen_water->Height() + 5;
+  gen_roughness = new SliderWidget( 0, xoff, yoff,
+                  w - xoff - 5, DEFAULT_SLIDER_SIZE, 1, 20, 6, 2,
+                  WIDGET_HSCROLL|WIDGET_ALIGN_LEFT, "_Roughness", this );
+
   yoff = h - f->Height() - 15;
-  ok = new ButtonWidget( B_ID_NMW_CREATE, 10, yoff, f->Width() * 10,
-       f->Height() + 10, 0, "_OK", this );
+  btn = new ButtonWidget( B_ID_NMW_CREATE, 10, yoff,
+        f->Width() * 10, f->Height() + 10, 0, "_OK", this );
+  btn->SetHook( this );
+
+  new ButtonWidget( GUI_CLOSE,
+      w - 10 - btn->w, yoff, btn->w, btn->h, 0, "_Cancel", this );
 
-  new ButtonWidget( GUI_CLOSE, w - 10 - ok->w, yoff, ok->w, ok->h,
-                    0, "_Cancel", this );
   Draw();
   Show();
 }
@@ -383,6 +417,12 @@ GUI_Status NewMissionWindow::WidgetActivated( Widget *button, Window *win ) {
       file = file_part( file );
       file.erase( file.length() - 6 );   // remove file name suffix
       tileset->SetString( file.c_str() );
+      view->CloseWindow( win );
+
+      if ( file == "default" ) gen_random->Enable();
+      else gen_random->Disable();
+      gen_random->Draw();
+      WidgetActivated( gen_random, this );
     }
     break; }
   case B_ID_UNITSET:
@@ -397,13 +437,78 @@ GUI_Status NewMissionWindow::WidgetActivated( Widget *button, Window *win ) {
       file = file_part( file );
       file.erase( file.length() - 6 );   // remove file name suffix
       unitset->SetString( file.c_str() );
+      view->CloseWindow( win );
     }
     break; }
+  case B_ID_RANDOM:
+    if ( button->Disabled() || !button->Clicked() ) {
+      gen_water->Disable();
+      gen_roughness->Disable();
+    } else {
+      gen_water->Enable();
+      gen_roughness->Enable();
+    }
+    gen_water->Draw();
+    gen_roughness->Draw();
+    Show();
+    break;
+  case B_ID_NMW_CREATE:
+    if ( hook ) {
+      mission = NewMission( GetMapSize(),
+                GetTileSet(), GetUnitSet() );
+      if ( mission ) {
+        if ( !gen_random->Disabled() && gen_random->Clicked() ) {
+          MapGenerator gen;
+          gen.Generate( mission->GetMap(),
+              gen_water->Level(), gen_roughness->Level() );
+        }
+        hook->WidgetActivated( button, this );
+      }
+    }
   }
 
   return GUI_OK;
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : NewMissionWindow::NewMission
+// DESCRIPTION: Create a new mission.
+// PARAMETERS : size - map size
+//              tset - tile set name
+//              uset - unit set name
+// RETURNS    : pointer to new mission or NULL on error
+////////////////////////////////////////////////////////////////////////
+
+Mission *NewMissionWindow::NewMission( const Point &size,
+         const string &tset, const string &uset ) {
+  Mission *ms = NULL;
+  TerrainSet *ts = new TerrainSet;
+  UnitSet *us = new UnitSet;
+
+  string tname( get_data_dir() + tset + ".tiles" );
+  File tfile( tname );
+
+  string uname( get_data_dir() + uset + ".units" );
+  File ufile( uname );
+
+  if ( !tfile.Open("rb") || ts->Load( tfile, tset.c_str() ) ) {
+    new NoteWindow( "Error", "Tile set not available", 0, view );
+  } else if ( !ufile.Open("rb") || us->Load( ufile, uset.c_str() ) ) {
+    new NoteWindow( "Error", "Unit set not available", 0, view );
+  } else {
+    ms = new Mission( size, ts, us );
+  }
+
+  if ( !ms ) {
+    delete ts;
+    delete us;
+  }
+
+  tfile.Close();
+  ufile.Close();
+  return ms;
+}
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : EdMissionSetupWindow::EdMissionSetupWindow
 // DESCRIPTION: This window allows modification of the basic mission
@@ -447,7 +552,14 @@ EdMissionSetupWindow::EdMissionSetupWindow( Mission &mission, View *view ) :
   wd = new CheckboxWidget( B_ID_CAMPAIGN, 5, wdy + (wdh - DEFAULT_CBW_SIZE)/2,
        DEFAULT_CBW_SIZE, DEFAULT_CBW_SIZE, mission.IsCampaign(),
        WIDGET_STYLE_NOBORDER|WIDGET_STYLE_GFX|WIDGET_ALIGN_RIGHT,
-       "Part of a _campaign", this );
+       "_Campaign Map", this );
+  wd->SetHook( this );
+
+  wdx = wd->LeftEdge() + wd->Width() + sfont->TextWidth("Campaign Map") + 15;
+  wd = new CheckboxWidget( B_ID_SKIRMISH, wdx, wd->TopEdge(),
+       DEFAULT_CBW_SIZE, DEFAULT_CBW_SIZE, mission.IsSkirmish(),
+       WIDGET_STYLE_NOBORDER|WIDGET_STYLE_GFX|WIDGET_ALIGN_RIGHT,
+       "S_kirmish Map", this );
   wd->SetHook( this );
 
   wdy += wdh + 10;
@@ -526,6 +638,10 @@ GUI_Status EdMissionSetupWindow::WidgetActivated( Widget *widget, Window *win )
     msg->SetText( n ? mission.GetMessage(n->ID()) : NULL );
     break;
 
+  case B_ID_SKIRMISH:
+    mission.SetSkirmish( widget->Clicked() );
+    break;
+
   case S_ID_SEQUEL:
     mission.SetSequel( static_cast<StringWidget *>(widget)->String() );
     break;
@@ -535,21 +651,25 @@ GUI_Status EdMissionSetupWindow::WidgetActivated( Widget *widget, Window *win )
     break;
 
   case B_ID_PLAYER1:
-  case B_ID_PLAYER2: {
+  case B_ID_PLAYER2:
     pid = widget->ID() - B_ID_PLAYER1;
     n = static_cast<ListSelectWindow *>(win)->Selected();
 
-    const char *name = mission.GetMessage(n->ID());
-    if ( name && strlen(name) > 30 ) {
-      new NoteWindow( "Error",
-                      "Player names must not be longer than 30 characters.",
-                      0, view );
+    if ( n ) {
+      const char *name = mission.GetMessage(n->ID());
+      if ( name && strlen(name) > 30 ) {
+        new NoteWindow( "Error",
+                        "Player names must not be longer than 30 characters.",
+                        0, view );
+      } else {
+        mission.GetPlayer(pid).SetNameID( n->ID() );
+        msg->SetText( name );
+      }
     } else {
-      mission.GetPlayer(pid).SetNameID( n ? n->ID() : -1 );
-      if ( showmsg->GetValue() == pid + 1 )
-        msg->SetText( name ? name : NULL );
+      mission.GetPlayer(pid).SetNameID( -1 );
+      msg->SetText( NULL );
     }
-    break; }
+    break;
 
   case B_ID_SET: {
     short msgid;
@@ -627,7 +747,7 @@ EdBuildingWindow::EdBuildingWindow( Building &bld, Mission &mission, View *view
 
   ButtonWidget *btn;
   btn = new ButtonWidget( B_ID_NAME, 5, yoff, sfont->Width() * 4 + 10, wdh,
-                          WIDGET_DEFAULT, "_Name", this ),
+                          0, "_Name", this ),
   btn->SetHook( this );
 
   xoff = 10 + btn->Width();
@@ -772,7 +892,7 @@ GUI_Status EdBuildingWindow::WidgetActivated( Widget *widget, Window *win ) {
   case B_ID_NAME_OK: {
     TLWNode *mn = static_cast<ListSelectWindow *>(win)->Selected();
 
-    const char *name = mission.GetMessage(mn->ID());
+    const char *name = (mn ? mission.GetMessage(mn->ID()) : NULL);
     if ( name && strlen(name) > 30 ) {
       new NoteWindow( "Error",
                       "Shop names must not be longer than 30 characters.",
@@ -799,9 +919,6 @@ GUI_Status EdBuildingWindow::WidgetActivated( Widget *widget, Window *win ) {
 
 EdUnitWindow::EdUnitWindow( Unit &unit, Mission &mission, View *view ) :
               Window(WIN_CENTER, view), u(unit), mission(mission), mv(0) { 
-  static const char *xp_labels[] = { "0", "1", "2", "3", "4", "5", "6", 0 };
-  static const char *size_labels[] = { "1", "2", "3", "4", "5", "6", 0 };
-
   // calculate dimensions
   SetSize( MIN(sfont->Width() * 50, view->Width()),
            MIN(sfont->Height() * 12, view->Height()) );
@@ -1324,3 +1441,250 @@ const UnitType *SelectUnitWindow::Selected( void ) const {
   return NULL;
 }
 
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : EdMsgWindow::EdMsgWindow
+// DESCRIPTION: This window offers a way to create new language catalogs
+//              and messages or modify existing ones.
+// PARAMETERS : locale - locale of the current mission
+//              view   - pointer to the window's view
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+EdMsgWindow::EdMsgWindow( Locale &locale, View *view ) :
+     Window(WIN_CENTER, view), locale(locale), cur_lang(0), cur_msg_id(-1) { 
+  // calculate dimensions
+  SetSize( MIN(sfont->Width() * 35 + 20, view->Width()),
+           MIN((sfont->Height() + 2) * 10 + 50, view->Height()) );
+
+  BuildLangsList();
+  if ( !l_langs.IsEmpty() ) {
+    cur_lang = static_cast<Language *>(static_cast<TLWNode *>(l_langs.Head())->UserData());
+    BuildMsgsList();
+    if ( !l_msgs.IsEmpty() ) cur_msg_id = 0;
+  }
+
+  ButtonWidget *btn;
+  unsigned short wdh = sfont->Height() + 8, wdw, wdy;
+
+  wdw = sfont->TextWidth("Languages") + 10;
+
+  w_langs = new TextListWidget( L_ID_LANGS,
+            wdw * 2 + 20, 10, w - 30 - wdw * 2, sfont->Height() + 10 + 2 * wdh,
+            &l_langs, l_langs.IsEmpty() ? -1 : 0, WIDGET_HSCROLLKEY, NULL, this );
+  w_langs->SetHook( this );
+
+  btn = new ButtonWidget( B_ID_LANG_NEW,
+            10, w_langs->TopEdge() + sfont->Height() + 5, wdw, wdh,
+            0, "_New", this );
+  btn->SetHook( this );
+
+  wdw = sfont->TextWidth("ID") + 5;
+  w_lid = new StringWidget( 0,
+          btn->LeftEdge() + wdw, btn->TopEdge() + wdh + 5,
+          btn->Width() - wdw, wdh,
+          NULL, 2, WIDGET_ALIGN_LEFT, "_ID", this );
+
+  btn = new ButtonWidget( B_ID_LANG_DELETE,
+            btn->LeftEdge() + btn->Width() + 5, btn->TopEdge(),
+            btn->Width(), wdh, 0, "_Delete", this );
+  btn->SetHook( this );
+
+  wdy = w_lid->TopEdge() + wdh + 10;
+  w_msgs = new TextListWidget( L_ID_MSGS,
+           5, wdy, w - 10, h - wdy - 2 * wdh - 15,
+           &l_msgs, cur_msg_id, WIDGET_VSCROLLKEY, NULL, this );
+  w_msgs->SetHook( this );
+
+  wdw = sfont->TextWidth("Text") + 10;
+  w_msg = new StringWidget( 0,
+          wdw, w_msgs->TopEdge() + w_msgs->Height() + 5, w - wdw - 5, wdh,
+          cur_msg_id == -1 ? NULL : cur_lang->GetMsg(cur_msg_id), 4096,
+          WIDGET_ALIGN_LEFT, "_Text", this );
+
+  btn = new ButtonWidget( B_ID_MSG_ADD,
+            w_msgs->LeftEdge(), w_msg->TopEdge() + wdh + 5, (w - 20) / 4, wdh,
+            0, "_Add", this );
+  btn->SetHook( this );
+  btn = new ButtonWidget( B_ID_MSG_MODIFY,
+            btn->LeftEdge() + btn->Width() + 5, btn->TopEdge(), btn->Width(), wdh,
+            0, "_Modify", this );
+  btn->SetHook( this );
+
+  btn = new ButtonWidget( GUI_CLOSE,
+            w - 5 - btn->Width(), btn->TopEdge(), btn->Width(), wdh,
+            0, "_OK", this );
+
+  Draw();
+  Show();
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : EdMsgWindow::WidgetActivated
+// DESCRIPTION: React to user actions.
+// PARAMETERS : widget - activated widget
+//              win    - window containing the widget
+// RETURNS    : GUI status
+////////////////////////////////////////////////////////////////////////
+
+GUI_Status EdMsgWindow::WidgetActivated( Widget *widget, Window *win ) {
+  TLWNode *n;
+
+  switch ( widget->ID() ) {
+  case L_ID_LANGS:
+    n = static_cast<TLWNode *>( w_langs->Selected() );
+    if ( n ) {
+      cur_lang = static_cast<Language *>( n->UserData() );
+      BuildMsgsList();
+      w_msgs->Update();
+      if ( cur_msg_id != -1 ) {
+        w_msgs->Select( cur_msg_id );
+        w_msg->SetString( cur_lang->GetMsg( cur_msg_id ) );
+      }
+      w_msgs->Draw();
+      w_msgs->Show();
+    }
+    break;
+  case L_ID_MSGS:
+    n = static_cast<TLWNode *>( w_msgs->Selected() );
+    if ( n ) {
+      w_msg->SetString( (char *)n->UserData() );
+      cur_msg_id = n->ID();
+    }
+    break;
+
+  case B_ID_LANG_NEW: {
+    Language l;
+
+    if ( l_langs.CountNodes() == 0 ) {
+      // add empty English catalog
+      l.SetID( CF_LANG_DEFAULT );
+    } else {
+      // copy English catalog to new language
+      const char *id = w_lid->String();
+      if ( id ) {
+        if ( locale.GetLanguage( id ) ) {
+          new NoteWindow( "Error", "This ID is already taken.", 0, view );
+          break;
+        } else {
+          const Language *def = locale.GetLanguage( CF_LANG_DEFAULT );
+          l.SetID( id );
+          for ( int i = 0; i < def->Size(); ++i )
+            l.AddMsg( def->GetMsg( i ) );
+        }
+      } else {
+        new NoteWindow( "Error", "Please enter an ID first.", 0, view );
+        break;
+      }
+    }
+    locale.AddLanguage( l );
+    BuildLangsList();
+    w_langs->Update();
+    w_langs->Draw();
+    w_langs->Show();
+    break; }
+  case B_ID_LANG_DELETE:
+    if ( cur_lang ) {
+      if ( (string(cur_lang->ID()) == CF_LANG_DEFAULT) && (l_langs.CountNodes() > 1) ) {
+        new NoteWindow( "Error", "You cannot delete the default catalog", 0, view );
+      } else {
+        locale.RemoveLanguage( *cur_lang );
+        BuildLangsList();
+        w_langs->Update();
+        w_langs->Draw();
+        w_langs->Show();
+
+        if ( w_langs->Selected() ) {
+          cur_lang = static_cast<Language *>(
+                     static_cast<TLWNode *>(w_langs->Selected())->UserData()
+                     );
+        } else cur_lang = NULL;
+        BuildMsgsList();
+        cur_msg_id = l_msgs.IsEmpty() ? -1 : 0;
+        w_msgs->Update();
+        w_msgs->Select( cur_msg_id );
+        w_msgs->Draw();
+        w_msgs->Show();
+        w_msg->SetString( cur_lang ? cur_lang->GetMsg( cur_msg_id ) : NULL );
+      }
+    }
+    break;
+  case B_ID_MSG_ADD:
+    if ( w_msg->String() ) {
+      // append the message to _all_ languages
+      for ( TLWNode *n = static_cast<TLWNode *>( l_langs.Head() );
+            n; n = static_cast<TLWNode *>( n->Next() ) ) {
+        Language *l = static_cast<Language *>( n->UserData() );
+        l->AddMsg( w_msg->String() );
+      }
+      BuildMsgsList();
+      w_msgs->Update();
+      if ( cur_msg_id == -1 ) w_msg->SetString( NULL );
+      else w_msgs->Select( cur_lang->Size() - 1 );
+      w_msgs->Draw();
+      w_msgs->Show();
+    }
+    break;
+  case B_ID_MSG_MODIFY:
+    if ( (cur_msg_id != -1) && w_msg->String() ) {
+      cur_lang->SetMsg( w_msg->String(), cur_msg_id );
+      BuildMsgsList();
+      w_msgs->Draw();
+      w_msgs->Show();
+    }
+    break;
+  }
+
+  return GUI_OK;
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : EdMsgWindow::Draw
+// DESCRIPTION: Draw the window.
+// PARAMETERS : -
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void EdMsgWindow::Draw( void ) {
+  Window::Draw();
+  DrawBox( Rect(5, 5, w - 10, w_langs->Height() + 10), BOX_CARVED );
+  sfont->Write( "Languages", this, 10, 10 );
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : EdMsgWindow::BuildLangsList
+// DESCRIPTION: Compile the list of available languages.
+// PARAMETERS : -
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void EdMsgWindow::BuildLangsList( void ) {
+  const std::map<const string, Language> &lib = locale.GetLibrary();
+  l_langs.Clear();
+
+  for ( map<const string, Language>::const_iterator iter = lib.begin();
+        iter != lib.end(); ++iter ) {
+    l_langs.InsertNodeSorted( new TLWNode( iter->second.ID(),
+            const_cast<Language *>(&iter->second) ) );
+  }
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : EdMsgWindow::BuildMsgsList
+// DESCRIPTION: Compile the list of messages in the current language.
+// PARAMETERS : -
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void EdMsgWindow::BuildMsgsList( void ) {
+  l_msgs.Clear();
+
+  if ( cur_lang ) {
+    const char *m;
+
+    for ( int i = 0; (m = cur_lang->GetMsg(i)) != 0; ++i ) {
+      l_msgs.AddTail( new TLWNode(string(m).substr(0, 30).c_str(), const_cast<char *>(m), i) );
+    }
+  }
+}
+
