@@ -1,5 +1,5 @@
 // CoMET - The Crimson Fields Map Editing Tool
-// Copyright (C) 2002-2004 Jens Granseuer
+// Copyright (C) 2002-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -78,25 +78,35 @@ class NewMissionWindow : public Window, public WidgetHook {
 public:
   NewMissionWindow( View *view );
 
-  void SetHook( WidgetHook *hook ) const { ok->SetHook(hook); }
+  void SetHook( WidgetHook *hook ) { this->hook = hook; }
 
+  Mission *GetMission( void ) const { return mission; }
+
+private:
+  Mission *NewMission( const Point &size, const string &tset,
+                   const string &uset );
   Point GetMapSize( void ) const;
   const char *GetTileSet( void ) const { return tileset->String(); }
   const char *GetUnitSet( void ) const { return unitset->String(); }
 
-private:
   GUI_Status WidgetActivated( Widget *button, Window *win );
 
   static const short B_ID_UNITSET;
   static const short B_ID_TILESET;
   static const short B_ID_UNITSET_OK;
   static const short B_ID_TILESET_OK;
+  static const short B_ID_RANDOM;
+
+  Mission *mission;
 
   NumberWidget *m_width;
   NumberWidget *m_height;
   StringWidget *tileset;
   StringWidget *unitset;
-  ButtonWidget *ok;
+  CheckboxWidget *gen_random;
+  SliderWidget *gen_water;
+  SliderWidget *gen_roughness;
+  WidgetHook *hook;
 };
 
 class EdMissionSetupWindow : public Window, public WidgetHook {
@@ -113,6 +123,7 @@ private:
   static const short B_ID_BRIEF2;
   static const short B_ID_INFO;
   static const short B_ID_CAMPAIGN;
+  static const short B_ID_SKIRMISH;
   static const short B_ID_CAMPAIGN_NAME;
   static const short B_ID_CAMPAIGN_INFO;
   static const short B_ID_SHOWMSG;
@@ -225,10 +236,9 @@ private:
   Mission &mission;
 };
 
-
 class EdUnitsWindow : public Window, public WidgetHook {
 public:
-  EdUnitsWindow( Mission &mission, MapObject &container,  View *view );
+  EdUnitsWindow( Mission &mission, MapObject &container, View *view );
 
 private:
   GUI_Status WidgetActivated( Widget *button, Window *win );
@@ -265,5 +275,34 @@ private:
   Mission &mission;
 };
 
+class EdMsgWindow : public Window, public WidgetHook {
+public:
+  EdMsgWindow( Locale &locale, View *view );
+
+private:
+  void Draw( void );
+  void BuildLangsList( void );
+  void BuildMsgsList( void );
+  GUI_Status WidgetActivated( Widget *button, Window *win );
+
+  static const short B_ID_LANG_NEW;
+  static const short B_ID_LANG_DELETE;
+  static const short B_ID_MSG_ADD;
+  static const short B_ID_MSG_MODIFY;
+  static const short L_ID_LANGS;
+  static const short L_ID_MSGS;
+
+  TextListWidget *w_langs;
+  TextListWidget *w_msgs;
+  StringWidget *w_msg;
+  StringWidget *w_lid;
+  Locale &locale;
+
+  TLWList l_langs;
+  TLWList l_msgs;
+  Language *cur_lang;
+  int cur_msg_id;
+};
+
 #endif	// _INCLUDE_ED_EXTWINDOW2_H
 
