@@ -1,5 +1,5 @@
 // CoMET - The Crimson Fields Map Editing Tool
-// Copyright (C) 2002-2004 Jens Granseuer
+// Copyright (C) 2002-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -28,10 +28,10 @@
 #include "globals.h"
 
 const char *event_labels[] = {
-      "Message", "Mining", "Score", "Next Map",
-      "Create Unit", "Manipulate Event", "Research", 0 };
+      "Message", "Mining", "Score", "Configure", "Create Unit",
+      "Manipulate Event", "Research", "Set Hex", "Set Timer", 0 };
 const char *etrigger_labels[] = {
-      "Turn", "Unit Destruction", "Shop Owner", "Unit Owner",
+      "Timer", "Unit Destruction", "Shop Owner", "Unit Owner",
       "Unit Position", "Handicap", 0 };
 
 #define P1_COLOR_LIGHT	0x00FEA604		// yellow
@@ -50,7 +50,7 @@ const char *etrigger_labels[] = {
 
 Mission::Mission( const Point &size, TerrainSet *ts, UnitSet *us ) :
                   p1(PLAYER_ONE), p2(PLAYER_TWO) {
-  flags = 0;
+  flags = GI_SKIRMISH;
   name = -1;
   campaign_name = -1;
   level_info = -1;
@@ -95,6 +95,7 @@ int Mission::Load( const char *filename ) {
     signed char mapid, musicid;
     string uset, tset;
     Building *b;
+    Event *e;
 
     // set last filename
     last_file_name = file_part( filename );
@@ -117,27 +118,31 @@ int Mission::Load( const char *filename ) {
     file.Read8();                   // current player - unused
     file.Read8();                   // turn phase - unused
 
-    len = file.Read16();            // load name of unit set
-    uset = file.ReadS( len );
+    p1.Load( file );
+    p2.Load( file );
 
-    len = file.Read16();            // load name of terrain set
-    tset = file.ReadS( len );
+    // load text messages
+    messages.Load( file );
 
     map.Load( file );               // load map
 
+    len = file.Read16();            // load name of unit set
+    uset = file.ReadS( len );
+
     unit_set = new UnitSet;
-    string usetname( get_data_dir() + uset + ".units" );
-    File ufile( usetname );
-    if ( !ufile.Open("rb") || unit_set->Load( ufile, uset.c_str() ) ) {
+    File ufile( uset + ".units" );
+    if ( !ufile.OpenData("rb") || unit_set->Load( ufile, uset.c_str() ) ) {
       cerr << "Error: Unit set '" << uset << "' not available." << endl;
       return -1;
     }
     ufile.Close();
 
+    len = file.Read16();            // load name of terrain set
+    tset = file.ReadS( len );
+
     terrain_set = new TerrainSet;
-    string tsetname( get_data_dir() + tset + ".tiles" );
-    File tfile( tsetname );
-    if ( !tfile.Open("rb") || terrain_set->Load( tfile, tset.c_str() ) ) {
+    File tfile( tset + ".tiles" );
+    if ( !tfile.OpenData("rb") || terrain_set->Load( tfile, tset.c_str() ) ) {
       cerr << "Error: Terrain set '" << tset << "' not available." << endl;
       return -1;
     }
@@ -146,9 +151,6 @@ int Mission::Load( const char *filename ) {
     map.SetUnitSet( unit_set );
     map.SetTerrainSet( terrain_set );
 
-    p1.Load( file );
-    p2.Load( file );
-
     len = file.Read16();                // load buildings
     for ( i = 0; i < len; ++i ) {
       b = new Building();
@@ -176,14 +178,11 @@ int Mission::Load( const char *filename ) {
 
     len = file.Read16();                // load events
     for ( i = 0; i < len; ++i ) {
-      Event *e = new Event();
+      e = new Event();
       e->Load( file );
       events.AddTail( e );
     }
 
-    // load text messages
-    messages.Load( file );
-
     Language internal_messages;
     internal_messages.ReadCatalog( file );
     SetSequel( internal_messages.GetMsg( mapid ) );
@@ -194,6 +193,20 @@ int Mission::Load( const char *filename ) {
           b; b = static_cast<Building *>(b->Next()) )
       b->SetName( GetMessage( b->NameID() ) );
 
+    // set special event properties
+    for ( e = static_cast<Event *>(events.Head());
+          e; e = static_cast<Event *>(e->Next()) ) {
+      if ( (e->Type() == EVENT_CONFIGURE) &&
+           (e->GetData(0) == 2) ) {    // set next map
+        const char *nmap = internal_messages.GetMsg( e->GetData(1) );
+        if ( nmap ) e->SetTmpBuf( nmap );
+        else {
+          e->SetTmpBuf("");
+          e->SetData(1, -1);
+        }
+      }
+    }
+
     len = file.Read16();
 
     // history - only present in saved games
@@ -213,7 +226,7 @@ int Mission::Load( const char *filename ) {
 
 int Mission::Save( const char *filename ) {
   File file( filename );
-  if ( !file.Open( "wb" ) ) {
+  if ( !file.Open( "wb", false ) ) {
     cerr << "Failed to save mission!" << endl;
     return -1;
   }
@@ -231,6 +244,22 @@ int Mission::Save( const char *filename ) {
     internal_messages.AddMsg( music );
   }
 
+  // get special event properties
+  Event *e;
+  for ( e = static_cast<Event *>( events.Head() );
+        e; e = static_cast<Event *>( e->Next() ) ) {
+    if ( (e->Type() == EVENT_CONFIGURE) && (e->GetData(0) == 2) ) { // next map
+      if ( e->GetTmpBuf() != "" ) {
+        short msgid = internal_messages.Find( e->GetTmpBuf() );
+        if ( msgid != -1 ) e->SetData( 1, msgid );
+        else {
+          e->SetData( 1, internal_messages.Size() );
+          internal_messages.AddMsg( e->GetTmpBuf() );
+        }
+      } else e->SetData( 1, -1 );
+    }
+  }
+
   // set last filename
   last_file_name = file_part( filename );
   num = last_file_name.rfind( '.' );
@@ -252,17 +281,19 @@ int Mission::Save( const char *filename ) {
   file.Write8( PLAYER_ONE );
   file.Write8( TURN_START );
 
+  p1.Save( file );             // save player data
+  p2.Save( file );
+
+  messages.Save( file );
+
+  map.Save( file );            // save map
+
   // save mission set info
   file.Write16( unit_set->GetName().length() );
   file.WriteS( unit_set->GetName() );
   file.Write16( terrain_set->GetName().length() );
   file.WriteS( terrain_set->GetName() );
 
-  map.Save( file );            // save map
-
-  p1.Save( file );             // save player data
-  p2.Save( file );
-
   // save buildings
   file.Write16( buildings.CountNodes() );
   for ( Building *b = static_cast<Building *>( buildings.Head() );
@@ -290,11 +321,10 @@ int Mission::Save( const char *filename ) {
 
   // save events
   file.Write16( events.CountNodes() );
-  for ( Event *e = static_cast<Event *>( events.Head() );
+  for ( e = static_cast<Event *>( events.Head() );
         e; e = static_cast<Event *>( e->Next() ) )
     e->Save( file );
 
-  messages.Save( file );
   internal_messages.WriteCatalog( file );
 
   file.Write16( 0 );        // turn history
@@ -322,7 +352,8 @@ int Mission::Export( const string &filename ) const {
   file << "mapwidth = " << map.Width() << '\n';
   file << "mapheight = " << map.Height() << '\n';
   file << "info = " << (short)level_info << '\n';
-  file << "campaign = " << (((flags & GI_CAMPAIGN) != 0) ? '1' : '0') << '\n';
+  file << "skirmish = " << (IsSkirmish() ? '1' : '0') << '\n';
+  file << "campaign = " << (IsCampaign() ? '1' : '0') << '\n';
   file << "campaignname = " << (short)campaign_name << '\n';
   file << "campaigninfo = " << (short)campaign_info << '\n';
 
@@ -354,7 +385,7 @@ int Mission::Export( const string &filename ) const {
 
   for ( Event *e = static_cast<Event *>(events.Head()); e;
         e = static_cast<Event *>(e->Next()) )
-    e->Export( file, unit_set );
+    e->Export( file, map );
 
   if ( messages.GetMsg(0) != 0 ) {
     const std::map<const string, Language> &lib = messages.GetLibrary();
@@ -391,10 +422,7 @@ Unit *Mission::CreateUnit( const UnitType *type, unsigned char pid, const Point
     units.AddTail( u );
 
     if ( !map.GetMapObject( pos ) ) map.SetUnit( u, pos );
-    else {
-      u->SetFlags( U_SHELTERED );
-      if ( u->IsMine() ) u->SetMoves( MCOST_MIN );
-    }
+    else u->SetFlags( U_SHELTERED );
   }
   return u;
 }
@@ -805,8 +833,22 @@ unsigned short Mission::ValidateEvent( Event &e, stringstream &errors ) const {
            << e.Name() << " (" << (short)e.ID() << ") event.\n\n";
   }
 
+  if ( (e.Dependency() != -1) && !GetEventByID(e.Dependency()) ) {
+    ++errs;
+    errors << "Warning: Event " << (short)e.ID()
+           << " depends on non-existing event " << (short)e.Dependency() << ".\n\n";
+  }
+
+  if ( (e.Discard() != -1) && !GetEventByID(e.Discard()) ) {
+    ++errs;
+    errors << "Warning: Event " << (short)e.ID()
+           << " discards non-existing event " << (short)e.Discard() << ".\n\n";
+  }
 
   switch ( e.Trigger() ) {
+  case ETRIGGER_HANDICAP:
+    break;
+
   case ETRIGGER_HAVE_BUILDING:
     if ( (e.GetTData(0) == -1) || !GetBuildingByID(e.GetTData(0)) ) {
       ++errs;
@@ -816,15 +858,14 @@ unsigned short Mission::ValidateEvent( Event &e, stringstream &errors ) const {
     break;
 
   case ETRIGGER_HAVE_UNIT:
-  case ETRIGGER_UNIT_POSITION:
-    if ( (e.GetTData(0) == -1) || !GetUnitByID(e.GetTData(0)) ) {
+    if ( (e.GetTData(0) < 0) || !GetUnitByID(e.GetTData(0)) ) {
       ++errs;
       errors << "Error: Trigger of " << e.Name() << " (" << (short)e.ID()
              << ") event targets non-existing unit with ID " << e.GetTData(0) << ".\n\n";
     }
     break;
 
-  case ETRIGGER_TURN:
+  case ETRIGGER_TIMER:
     break;
 
   case ETRIGGER_UNIT_DESTROYED:
@@ -845,6 +886,18 @@ unsigned short Mission::ValidateEvent( Event &e, stringstream &errors ) const {
     }
     break;
 
+  case ETRIGGER_UNIT_POSITION:
+    if ( (e.GetTData(0) > 0) && !GetUnitByID(e.GetTData(0)) ) {
+      ++errs;
+      errors << "Error: Trigger of " << e.Name() << " (" << (short)e.ID()
+             << ") event targets non-existing unit with ID " << e.GetTData(0) << ".\n\n";
+    } else if ( (e.GetTData(0) < -1) && (-e.GetTData(0) - 2 >= unit_set->NumUT()) ) {
+      ++errs;
+      errors << "Error: Trigger of " << e.Name() << " (" << (short)e.ID()
+             << ") event targets invalid unit type with ID " << -e.GetTData(0) - 2 << ".\n\n";
+    }
+    break;
+
   default:
     ++errs;
     errors << "Warning: Unknown event trigger " << e.TriggerName() << " for "
@@ -853,8 +906,12 @@ unsigned short Mission::ValidateEvent( Event &e, stringstream &errors ) const {
 
 
   switch ( e.Type() ) {
-  case EVENT_CREATE_UNIT:
-    break;
+  case EVENT_CONFIGURE:
+    if ( (e.GetData(1) != -1) && !GetMessage(e.GetData(1)) ) {
+    ++errs;
+    errors << "Error: Invalid message " << e.GetData(1) << " for "
+           << e.Name() << " (" << (short)e.ID() << ") event.\n\n";
+  }
 
   case EVENT_MINING:
   case EVENT_RESEARCH:
@@ -873,14 +930,9 @@ unsigned short Mission::ValidateEvent( Event &e, stringstream &errors ) const {
     }
     break;
 
-  case EVENT_NEXT_MAP:
-    if ( (e.GetData(0) != -1) && !GetMessage(e.GetData(0)) ) {
-    ++errs;
-    errors << "Error: Invalid mission file name " << e.GetData(0) << " for "
-           << e.Name() << " (" << (short)e.ID() << ") event.\n\n";
-  }
-
+  case EVENT_CREATE_UNIT:
   case EVENT_MESSAGE:
+  case EVENT_SET_HEX:
     break;
   case EVENT_SCORE:
     if ( (e.GetData(1) != -1) && !GetMessage(e.GetData(1)) ) {
@@ -899,6 +951,22 @@ unsigned short Mission::ValidateEvent( Event &e, stringstream &errors ) const {
     }
     break;
 
+  case EVENT_SET_TIMER:
+    if ( (e.GetData(0) == -1) || !GetEventByID(e.GetData(0)) ) {
+      ++errs;
+      errors << "Error: " << e.Name() << " (" << (short)e.ID()
+             << ") event targets non-existing event with ID " << e.GetData(0) << ".\n\n";
+    } else {
+      Event *tev = GetEventByID(e.GetData(0));
+      if ( tev->Trigger() != ETRIGGER_TIMER ) {
+        ++errs;
+        errors << "Error: " << e.Name() << " (" << (short)e.ID()
+               << ") event targets event that has no "
+               << etrigger_labels[ETRIGGER_TIMER] << " trigger.\n\n";
+      }
+    }
+    break;
+
   default:
     ++errs;
     errors << "Warning: Unknown event type " << e.Name() << " for event ID "
@@ -1023,9 +1091,9 @@ int Player::Export( ofstream &file ) const {
 ////////////////////////////////////////////////////////////////////////
 // NAME       : Event::Event
 // DESCRIPTION: Create a new event.
-// PARAMETERS : id       - unique event identifier
-//              type     - event type
-//              trigger  - event trigger type
+// PARAMETERS : id      - unique event identifier
+//              type    - event type
+//              trigger - event trigger type
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
@@ -1034,6 +1102,7 @@ Event::Event( unsigned char id, unsigned char type, unsigned char trigger ) {
   e_type = type;
   e_trigger = trigger;
   e_depend = -1;
+  e_discard = -1;
   e_title = -1;
   e_message = -1;
   e_flags = 0;
@@ -1041,42 +1110,55 @@ Event::Event( unsigned char id, unsigned char type, unsigned char trigger ) {
 
   // set event data defaults
   switch ( e_type ) {
-  case EVENT_MESSAGE:
-    e_data[0] = e_data[1] = e_data[2] = 0;
-    break;
-  case EVENT_MINING:
-    e_data[0] = -1;        // change mining in shop -1
-    e_data[1] = 0;         // set crystals to this amount
-    e_data[2] = 0;         // set storage to absolute amount
-    break;
-  case EVENT_SCORE:
-    e_data[0] = 100;       // award 100 points
-    e_data[1] = e_data[2] = -1; // don't show a message to the loser
-    break;
-  case EVENT_NEXT_MAP:
-    e_data[0] = -1;       // prepare map with this name
-    e_data[1] = e_data[2] = 0;
+  case EVENT_CONFIGURE:
+    e_data[0] = 0;         // set player 1 briefing
+    e_data[1] = -1;        // no briefing available
+    e_data[2] = 0;
     break;
   case EVENT_CREATE_UNIT:
     e_data[0] = 0;        // build first unit type in set
     e_data[1] = 0;        // build at position (0, 0)
-    e_data[2] = 0;
-    break;
-  case EVENT_RESEARCH:
-    e_data[0] = -1;      // make unit available in shop -1
-    e_data[1] = 0;       // research first unit in set
-    e_data[2] = 0;
+    e_data[2] = NORTH | (MAX_GROUP_SIZE << 3) | (0 << 6);
+                          // face northwards, MAX_GROUP_SIZE, no xp
     break;
   case EVENT_MANIPULATE_EVENT:
     e_data[0] = -1;      // modify event -1
     e_data[1] = 0;       // flags affected
     e_data[2] = 0;       // set selected flags
     break;
+  case EVENT_MESSAGE:
+    e_data[0] = -1;      // no default focus
+    e_data[1] = e_data[2] = 0;
+    break;
+  case EVENT_MINING:
+    e_data[0] = -1;        // change mining in shop -1
+    e_data[1] = 0;         // set crystals to this amount
+    e_data[2] = 0;         // set storage to absolute amount
+    break;
+  case EVENT_RESEARCH:
+    e_data[0] = -1;      // research in shop -1
+    e_data[1] = 0;       // research first unit in set
+    e_data[2] = 0;       // allow production of unit
+    break;
+  case EVENT_SCORE:
+    e_data[0] = 100;       // award 100 points
+    e_data[1] = e_data[2] = -1; // don't show a message to the loser
+    break;
+  case EVENT_SET_HEX:
+    e_data[0] = 0;       // set to first tile
+    e_data[1] = 0;       // change hex at (0, 0)
+    e_data[2] = 0;
+    break;
+  case EVENT_SET_TIMER:
+    e_data[0] = -1;      // invalid event
+    e_data[1] = 0;       // set timer to 0
+    e_data[2] = 2;       // relative to current configuration
+    break;
   }
 
   switch ( e_trigger ) {
-  case ETRIGGER_TURN:
-    e_tdata[0] = 0;     // activate on first turn
+  case ETRIGGER_TIMER:
+    e_tdata[0] = 0;     // activate at beginning of first turn
     e_tdata[1] = e_tdata[2] = 0;
     break;
   case ETRIGGER_UNIT_DESTROYED:
@@ -1091,15 +1173,17 @@ Event::Event( unsigned char id, unsigned char type, unsigned char trigger ) {
     e_tdata[2] = -1;          // activate any turn if condition is true
     break;
   case ETRIGGER_UNIT_POSITION:
-    e_tdata[0] = -1;   // require no specific unit
-    e_tdata[1] = 0;    // x coordinate of target hex
-    e_tdata[2] = 0;    // y coordinate of target hex
+    e_tdata[0] = -1;         // require no specific unit
+    e_tdata[1] = 0;          // coordinates of target hex (0/0)
+    e_tdata[2] = PLAYER_ONE; // player required to control the unit
     break;
   case ETRIGGER_HANDICAP:
     e_tdata[0] = HANDICAP_NONE|HANDICAP_P1|HANDICAP_P2; // trigger always
     e_tdata[1] = e_tdata[2] = 0;
     break;
   }
+
+  e_tmpbuf = "";
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -1116,6 +1200,7 @@ int Event::Load( File &file ) {
   e_type = file.Read8();
   e_trigger = file.Read8();
   e_depend = file.Read8();
+  e_discard = file.Read8();
   for ( i = 0; i < 3; ++i ) e_tdata[i] = file.Read16();
   for ( i = 0; i < 3; ++i ) e_data[i] = file.Read16();
   e_title = file.Read16();
@@ -1139,6 +1224,7 @@ int Event::Save( File &file ) const {
   file.Write8( e_type );
   file.Write8( e_trigger );
   file.Write8( e_depend );
+  file.Write8( e_discard );
   for ( i = 0; i < 3; ++i ) file.Write16( e_tdata[i] );
   for ( i = 0; i < 3; ++i ) file.Write16( e_data[i] );
   file.Write16( e_title );
@@ -1152,28 +1238,31 @@ int Event::Save( File &file ) const {
 // NAME       : Event::Export
 // DESCRIPTION: Save the event to a plain text file.
 // PARAMETERS : file - data file descriptor
-//              uset - current unit set
+//              map  - current map
 // RETURNS    : 0 on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int Event::Export( ofstream &file, const UnitSet *uset ) const {
-  const char *type[] = { "message", "mining", "score",
-        "nextmap", "createunit", "manipulateevent", "research" };
+int Event::Export( ofstream &file, const Map &map ) const {
+  const char *type[] = { "message", "mining", "score", "configure",
+        "createunit", "manipulateevent", "research", "sethex", "settimer" };
+  Point loc;
+  const UnitSet *uset = map.GetUnitSet();
 
   file << "[event]\n";
   file << "id = " << (short)e_id << '\n';
   file << "type = " << type[e_type-1] << '\n';
   file << "player = " << (e_player + 1) << '\n';
   if ( e_depend != -1 )  file << "depend = " << (short)e_depend << '\n';
+  if ( e_discard != -1 )  file << "discard = " << (short)e_discard << '\n';
   if ( e_title != -1 )   file << "title = " << e_title << '\n';
   if ( e_message != -1 ) file << "message = " << e_message << '\n';
   if ( e_flags != 0 )    file << "flags = " << e_flags << '\n';
 
   file << "trigger = ";
   switch ( e_trigger ) {
-  case ETRIGGER_TURN:
-    file << "turn\n";
-    file << "tturn = " << e_tdata[0] << '\n';
+  case ETRIGGER_TIMER:
+    file << "timer\n";
+    file << "ttime = " << e_tdata[0] << '\n';
     break;
   case ETRIGGER_UNIT_DESTROYED:
     file << "unitdestroyed\n";
@@ -1184,19 +1273,22 @@ int Event::Export( ofstream &file, const UnitSet *uset ) const {
     file << "havebuilding\n";
     file << "tbuilding = " << e_tdata[0] << '\n';
     file << "towner = " << e_tdata[1] + 1 << '\n';
-    file << "tturn = " << e_tdata[2] << '\n';
+    file << "ttime = " << e_tdata[2] << '\n';
     break;
   case ETRIGGER_HAVE_UNIT:
     file << "haveunit\n";
     file << "tunit = " << e_tdata[0] << '\n';
     file << "towner = " << e_tdata[1] + 1 << '\n';
-    file << "tturn = " << e_tdata[2] << '\n';
+    file << "ttime = " << e_tdata[2] << '\n';
     break;
   case ETRIGGER_UNIT_POSITION:
     file << "unitposition\n";
-    file << "tunit = " << e_tdata[0] << '\n';
-    file << "txpos = " << e_tdata[1] << '\n';
-    file << "typos = " << e_tdata[2] << '\n';
+    file << "tunit = ";
+    if ( e_tdata[0] >= -1 ) file << e_tdata[0] << '\n';
+    else file << uset->GetUnitInfo(-e_tdata[0] - 2)->Name() << '\n';
+    file << "towner = " << e_tdata[2] + 1 << '\n';
+    loc = map.Index2Hex( e_tdata[1] );
+    file << "tpos = " << loc.x << '/' << loc.y << '\n';
     break;
   case ETRIGGER_HANDICAP:
     file << "handicap\n";
@@ -1205,6 +1297,41 @@ int Event::Export( ofstream &file, const UnitSet *uset ) const {
   }
 
   switch ( e_type ) {
+  case EVENT_CONFIGURE:
+    file << "setting = ";
+    switch ( e_data[0] ) {
+    case 0: file << "briefing1"; break;
+    case 1: file << "briefing2"; break;
+    case 2: file << "nextmap"; break;
+    }
+    file << '\n' << "value = ";
+    switch ( e_data[0] ) {
+    case 0:
+    case 1: file << e_data[1]; break;
+    case 2: file << e_tmpbuf; break;
+    }
+    file << '\n';
+    break;
+  case EVENT_CREATE_UNIT:
+    file << "unit = " << uset->GetUnitInfo(e_data[0])->Name() << '\n';
+    loc = map.Index2Hex( e_data[1] );
+    file << "pos = " << loc.x << '/' << loc.y << '\n';
+    file << "face = " << (e_data[2] & 0x07) << '\n';
+    if ( ((e_data[2] & 0x38) >> 3) != MAX_GROUP_SIZE )
+      file << "size = " << ((e_data[2] & 0x38) >> 3) << '\n';
+    if ( ((e_data[2] & 0x1C0) >> 6) != 0 )
+      file << "xp = " << ((e_data[2] & 0x1C0) >> 6) << '\n';
+    break;
+  case EVENT_MANIPULATE_EVENT:
+    file << "event = " << e_data[0] << '\n';
+    file << "eflags = " << e_data[1] << '\n';
+    file << "action = " << e_data[2] << '\n';
+    break;
+  case EVENT_MESSAGE:
+    loc = map.Index2Hex( e_data[0] );
+    if ( loc != Point(-1,-1) )
+      file << "pos = " << loc.x << '/' << loc.y << '\n';
+    break;
   case EVENT_MINING:
     file << "building = " << e_data[0] << '\n';
     file << "crystals = " << e_data[1] << '\n';
@@ -1213,24 +1340,22 @@ int Event::Export( ofstream &file, const UnitSet *uset ) const {
   case EVENT_RESEARCH:
     file << "building = " << e_data[0] << '\n';
     file << "unit = " << uset->GetUnitInfo(e_data[1])->Name() << '\n';
+    if ( e_data[2] != 0 ) file << "action = " << e_data[2] << '\n';
     break;
   case EVENT_SCORE:
     file << "success = " << e_data[0] << '\n';
     file << "othermsg = " << e_data[1] << '\n';
     file << "othertitle = " << e_data[2] << '\n';
     break;
-  case EVENT_CREATE_UNIT:
-    file << "unit = " << uset->GetUnitInfo(e_data[0])->Name() << '\n';
-    file << "xpos = " << e_data[1] << '\n';
-    file << "ypos = " << e_data[2] << '\n';
+  case EVENT_SET_HEX:
+    file << "tile = " << e_data[0] << '\n';
+    loc = map.Index2Hex( e_data[1] );
+    file << "pos = " << loc.x << '/' << loc.y << '\n';
     break;
-  case EVENT_NEXT_MAP:
-    file << "map = " << e_data[0] << '\n';
-    break;
-  case EVENT_MANIPULATE_EVENT:
+  case EVENT_SET_TIMER:
     file << "event = " << e_data[0] << '\n';
-    file << "flags = " << e_data[1] << '\n';
-    file << "action = " << e_data[3] << '\n';
+    file << "time = " << e_data[1] << '\n';
+    file << "offset = " << e_data[2] << '\n';
     break;
   }
 
