@@ -1,5 +1,5 @@
 // CoMET - The Crimson Fields Map Editing Tool
-// Copyright (C) 2002-2004 Jens Granseuer
+// Copyright (C) 2002-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -68,7 +68,7 @@ public:
 
   int Load( File &file );
   int Save( File &file ) const;
-  int Export( ofstream &file, const UnitSet *uset ) const;
+  int Export( ofstream &file, const Map &map ) const;
 
   unsigned char ID( void ) const { return e_id; }
   void SetID( unsigned char id ) { e_id = id; }
@@ -91,6 +91,8 @@ public:
        { e_flags ^= flags; }
   signed char Dependency( void ) const { return e_depend; }
   void SetDependency( signed char dep ) { e_depend = dep; }
+  signed char Discard( void ) const { return e_discard; }
+  void SetDiscard( signed char ev ) { e_discard = ev; }
 
   short GetData( unsigned short index ) const { return e_data[index]; }
   void SetData( unsigned short index, short value )
@@ -99,22 +101,27 @@ public:
   void SetTData( unsigned short index, short value )
        { e_tdata[index] = value; }
 
+  void SetTmpBuf( const string &s ) { e_tmpbuf = s; }
+  const string &GetTmpBuf( void ) const { return e_tmpbuf; }
+
 private:
   unsigned char e_id;
   unsigned char e_type;
   unsigned char e_trigger;
   signed char e_depend;
+  signed char e_discard;
   short e_tdata[3];
   short e_data[3];
   short e_title;
   short e_message;
   unsigned short e_flags;
   unsigned char e_player;
+  string e_tmpbuf;
 };
 
 class Mission {
 public:
-  Mission( void ) : p1(PLAYER_ONE), p2(PLAYER_TWO),
+  Mission( void ) : flags(GI_SKIRMISH), p1(PLAYER_ONE), p2(PLAYER_TWO),
                     unit_set(0), terrain_set(0) {}
   Mission( const Point &size, TerrainSet *ts, UnitSet *us );
   ~Mission( void );
@@ -133,6 +140,7 @@ public:
   void SetMusic( const char *track )
     { track ? music = track : music.erase(); }
 
+  TerrainSet &GetTerrainSet( void ) const { return *terrain_set; }
   const UnitSet &GetUnitSet( void ) const { return *unit_set; }
   signed char GetLevelInfoMsg( void ) const { return level_info; }
   void SetLevelInfoMsg( signed char msg ) { level_info = msg; }
@@ -177,6 +185,9 @@ public:
   bool IsCampaign( void ) const { return (flags & GI_CAMPAIGN) != 0; }
   void SetCampaign( bool c )
     { c ? flags |= GI_CAMPAIGN : flags &= ~(GI_CAMPAIGN); }
+  bool IsSkirmish( void ) const { return (flags & GI_SKIRMISH) != 0; }
+  void SetSkirmish( bool s )
+    { s ? flags |= GI_SKIRMISH : flags &= ~(GI_SKIRMISH); }
 
 protected:
   unsigned short GetUnitID( void ) const;
