@@ -1,5 +1,5 @@
 // CoMET - The Crimson Fields Map Editing Tool
-// Copyright (C) 2002-2004 Jens Granseuer
+// Copyright (C) 2002-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -70,7 +70,6 @@ public:
   unsigned short ID( void ) const { return u_id; }
   void SetID( unsigned short id ) { u_id = id; }
   unsigned short Image( void ) const { return BaseImage() + u_facing; }
-  void SetMoves( unsigned char speed ) { u_moves = speed; }
   const char *Name( void ) const { return u_type->Name(); }
   unsigned char Owner( void ) const { return u_pid; }
   void SetOwner( unsigned char pid ) { u_pid = pid; }
@@ -95,7 +94,6 @@ public:
   bool IsMine( void ) const { return (u_flags & U_MINE) != 0; }
   bool IsTransport( void ) const { return (u_flags & U_TRANSPORT) != 0; }
   bool IsMedic( void ) const { return (u_flags & U_MEDIC) != 0; }
-  bool IsPioneer( void ) const { return (u_flags & U_PIONEERS) != 0; }
   bool IsMinesweeper( void ) const { return (u_flags & U_MINESWEEPER) != 0; }
 
   bool IsSlow( void ) const { return (u_flags & U_SLOW) != 0; }
@@ -112,7 +110,6 @@ protected:
   unsigned long u_flags;
   unsigned short u_id;
   
-  unsigned char u_moves;
   unsigned char u_facing;       // direction
   unsigned char u_group;        // group size
   unsigned char u_xp;           // experience
