@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -346,7 +346,8 @@ CycleWidget::CycleWidget( short id, short x, short y, unsigned short w,
 
   if ( !(flags & (WIDGET_ALIGN_CENTER|WIDGET_ALIGN_LEFT|
                   WIDGET_ALIGN_RIGHT|WIDGET_ALIGN_TOP)) )
-    this->flags |= WIDGET_ALIGN_LEFT;
+    // button widget defaults to center which we must revert
+    this->flags ^= WIDGET_ALIGN_LEFT|WIDGET_ALIGN_CENTER;
 
   image[0] = Image( window->GetView()->GetSystemIcons(),
                     GFX_CYCLE_X, GFX_CYCLE_Y,
@@ -392,16 +393,74 @@ void CycleWidget::Draw( void ) {
 }
 
 ////////////////////////////////////////////////////////////////////////
-// NAME       : CycleWidget::Release
-// DESCRIPTION: Update the button label and redraw the widget.
+// NAME       : CycleWidget::MouseDown
+// DESCRIPTION: Show the widget in depressed state if it was selected.
+// PARAMETERS : button - SDL_MouseButtonEvent received from the event
+//                       handler
+// RETURNS    : GUI_OK
+////////////////////////////////////////////////////////////////////////
+
+GUI_Status CycleWidget::MouseDown( const SDL_MouseButtonEvent &button ) {
+  if ( ((button.button == SDL_BUTTON_LEFT) ||
+       (button.button == SDL_BUTTON_RIGHT)) &&
+       Contains( button.x - surface->LeftEdge(),
+                 button.y - surface->TopEdge() ) ) {
+    up = (button.button == SDL_BUTTON_LEFT);
+    Push();
+  } else if ( Clicked() ) Release();
+  return GUI_OK;
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : CycleWidget::MouseUp
+// DESCRIPTION: Release the button and activate it if the mouse pointer
+//              is still over it.
+// PARAMETERS : button - SDL_MouseButtonEvent received from the event
+//                       handler
+// RETURNS    : GUI status
+////////////////////////////////////////////////////////////////////////
+
+GUI_Status CycleWidget::MouseUp( const SDL_MouseButtonEvent &button ) {
+  if ( Clicked() && 
+       ((button.button == SDL_BUTTON_LEFT) ||
+        (button.button == SDL_BUTTON_RIGHT)) ) {
+    bool hit = Contains( button.x - surface->LeftEdge(),
+                         button.y - surface->TopEdge() );
+    if ( hit ) CycleValue();
+    Release();
+    if ( hit ) return Activate();
+  }
+  return GUI_OK;
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : CycleWidget::KeyUp
+// DESCRIPTION: Activate widget if correct key was released.
+// PARAMETERS : key - SDL_keysym received from the event handler
+// RETURNS    : GUI status
+////////////////////////////////////////////////////////////////////////
+
+GUI_Status CycleWidget::KeyUp( const SDL_keysym &key ) {
+  if ( Clicked() && ((key.sym == this->key) ||
+       ((flags & WIDGET_DEFAULT) && (key.sym == SDLK_RETURN))) ) {
+    CycleValue();
+    Release();
+    return Activate();
+  }
+  return GUI_OK;
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : CycleWidget::CycleValue
+// DESCRIPTION: Update the widget value according to the button pressed.
 // PARAMETERS : -
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-void CycleWidget::Release( void ) {
-  val = (val + 1) % (maxval + 1);
-
-  ButtonWidget::Release();
+void CycleWidget::CycleValue( void ) {
+  if (up) val = (val + 1) % (maxval + 1);
+  else if ( val == 0 ) val = maxval;
+  else --val;
 }
 
 ////////////////////////////////////////////////////////////////////////
