@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "SDL_zlib.h"
 #include "fileio.h"
 #include "globals.h"      // for CF_SHORTNAME
 
@@ -190,21 +191,66 @@ void File::Close( void ) {
 ////////////////////////////////////////////////////////////////////////
 // NAME       : File::Open
 // DESCRIPTION: Open the file.
-// PARAMETERS : opts - file access descriptors (r, w, b)
+// PARAMETERS : mode       - file access descriptors (r, w, b)
+//              compressed - whether to open for compression. This
+//                           parameter should only be used when opening
+//                           a file for writing. Default is true.
 // RETURNS    : TRUE on success, FALSE on error
 ////////////////////////////////////////////////////////////////////////
 
-bool File::Open( const char *opts ) {
+bool File::Open( const char *mode, bool compressed /* = true */ ) {
   bool rc = false;
 
   if ( !fh ) {
-    fh = SDL_RWFromFile( name.c_str(), opts );
+    if ( compressed ) fh = SDL_RWFromGzip( name.c_str(), mode );
+    else fh = SDL_RWFromFile( name.c_str(), mode );
     rc = (fh != 0);
   }
 
   return rc;
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : File::OpenData
+// DESCRIPTION: Open the file. The "path" parameter passed to the
+//              constructor is treated as a path relative to the data
+//              (sub)directory used by the game. This method first tries
+//              to open the file from the user's home directory (if
+//              available), and only if this attempt fails the file is
+//              loaded from the system data directory. After a
+//              successful call, File::Name() returns the full path to
+//              the file.
+// PARAMETERS : subdir     - data subdirectory to look in
+//              mode       - file access descriptors (r, w, b)
+//              compressed - whether to open for compression. This
+//                           parameter should only be used when opening
+//                           a file for writing. Default is true.
+// RETURNS    : TRUE on success, FALSE on error
+////////////////////////////////////////////////////////////////////////
+
+bool File::OpenData( const char *mode, const string &subdir /* = "" */,
+                     bool compressed /* = true */ ) {
+  bool rc = false;
+
+  if ( !fh ) {
+    string local( name );
+    name = get_home_dir();
+    if ( !name.empty() ) {
+      append_path( name, subdir );
+      append_path( name, local );
+      rc = Open( mode, compressed );
+    }
+
+    if ( !rc ) {
+      name = get_data_subdir( subdir );
+      append_path( name, local );
+      rc = Open( mode, compressed );
+    }
+  }
+
+  return rc;
+}
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : File::Exists
 // DESCRIPTION: Check whether the filename is already in use.
@@ -322,8 +368,7 @@ void create_config_dir( void ) {
   if ( !cfdir.IsValid() ) make_dir( confd.c_str() );
 
   // now check for the saved games directory
-  append_path_delim( confd );
-  confd.append( "games" );
+  append_path( confd, "games" );
 
   Directory svdir( confd.c_str() );
   if ( !svdir.IsValid() ) make_dir( confd.c_str() );
@@ -450,8 +495,7 @@ string get_save_dir( void ) {
 
 string get_data_subdir( const string &sub ) {
   string d( get_data_dir() );
-  append_path_delim( d );
-  d.append( sub );
+  append_path( d, sub );
   append_path_delim( d );
   return d;
 }
@@ -538,3 +582,17 @@ string file_part( const string &path ) {
   return path.substr( pathend + 1 );
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : append_path
+// DESCRIPTION: Add another subpath to an existing path. Also add path
+//              delimiter if required.
+// PARAMETERS : path - existing path to be extended
+//              sub  - subpath to be added
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void append_path( string &path, const string &sub ) {
+  append_path_delim( path );
+  path.append( sub );
+}
+
