@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -202,8 +202,19 @@ GUI_Status ListWidget::WidgetActivated( Widget *widget, Window *win ) {
 ////////////////////////////////////////////////////////////////////////
 
 GUI_Status ListWidget::MouseDown( const SDL_MouseButtonEvent &button ) {
+  SDL_MouseButtonEvent mybutton = button;
+  if ( ((button.button == SDL_BUTTON_WHEELUP) ||
+       (button.button == SDL_BUTTON_WHEELDOWN)) &&
+       Contains( button.x - surface->LeftEdge(), button.y - surface->TopEdge() ) ) {
+    // reroute all wheel events to the scroller
+    SliderWidget *s = static_cast<SliderWidget *>( GetWidget(0) );
+    if ( s ) {
+      mybutton.x = s->LeftEdge() + surface->LeftEdge();
+      mybutton.y = s->TopEdge() + surface->TopEdge();
+    }
+  }
 
-  GUI_Status rc = CompositeWidget::MouseDown( button );
+  GUI_Status rc = CompositeWidget::MouseDown( mybutton );
 
   if ( (rc == GUI_OK) && (button.button == SDL_BUTTON_LEFT) ) {
     short mx = button.x - surface->LeftEdge();
