@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -77,7 +77,7 @@ public:
   unsigned char ID( void ) const { return ut_typeid; }
   signed char Portrait( void ) const { return ut_portrait; }
 
-private:
+protected:
   unsigned short ut_terrain;		// wherever it may roam...
   unsigned short ut_image;
   unsigned short ut_flags;
@@ -117,7 +117,7 @@ public:
   Surface *GetTiles( void ) { return &tiles; }
   SoundEffect *GetSound( unsigned short sfxid ) const;
   Surface *GetPortrait( unsigned char ptid ) const { return &portraits[ptid]; }
-  const char *UnitName( unsigned short utid )
+  const char *UnitName( unsigned short utid ) const
              { return unit_names.GetMsg( utid ); }
 
   void SetLocale( const string &lang );
