@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -273,7 +273,7 @@ bool MapView::HexVisible( const Point &hex ) const {
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : MapView::UpdateHex
-// DESCRIPTION: Redraw a single hex and update the display.
+// DESCRIPTION: Redraw a single hex.
 // PARAMETERS : hex - hex to update
 // RETURNS    : Rect describing the surface area that has been updated
 //              and needs to be refreshed
@@ -325,7 +325,7 @@ Rect MapView::UpdateHex( const Point &hex ) {
 ////////////////////////////////////////////////////////////////////////
 
 void MapView::DrawHex( unsigned short n, Surface *src, Surface *dest,
-                       short px, short py, const Rect &clip ) const {
+                       short px, short py, const Rect &clip ) {
   // set up destination
   Rect dstrect( px, py, GFX_WIDTH, GFX_HEIGHT );
   if ( dstrect.x + dstrect.w < clip.x ) return;
