@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -52,6 +52,7 @@
 #define MV_DISABLE_FOG		0x0004	// don't show fog
 #define MV_DISABLE_CURSOR	0x0008	// don't show cursor
 #define MV_ENABLE_UNIT_STATS	0x0010  // overlay units with stats (health, xp)
+#define MV_DIRTY		0x8000  // must be redrawn
 
 class MapView : public Rect {
 public:
@@ -85,8 +86,8 @@ public:
   void DrawMap( short x, short y, unsigned short w, unsigned short h,
                 Surface *dest, short dx, short dy ) const;
   void DrawFog( Surface *dest, short px, short py, const Rect &clip ) const;
-  void DrawHex( unsigned short n, Surface *src, Surface *dest,
-                short px, short py, const Rect &clip ) const;
+  static void DrawHex( unsigned short n, Surface *src, Surface *dest,
+                       short px, short py, const Rect &clip );
   void DrawUnit( unsigned short n, Surface *dest,
                 short px, short py, const Rect &clip ) const
                { DrawHex( n, utiles, dest, px, py, clip ); }
