@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 ////////////////////////////////////////////////////////////////////////////
 
 #include "surface.h"
-#include "misc.h"
 #include "globals.h"
 
 const unsigned char Surface::RAW_DATA_TRANSPARENT = 0x01;
@@ -309,69 +308,6 @@ void Surface::FillPattern( short x, short y, unsigned short w, unsigned short h,
   SetClipRect( clip );
 }
 
-////////////////////////////////////////////////////////////////////////
-// NAME       : Surface::Colorize
-// DESCRIPTION: Change all non-transparent pixels in the specified part
-//              of the surface to one color.
-// PARAMETERS : x   - left edge of rectangle
-//              y   - top edge of rectangle
-//              w   - width of rectangle
-//              h   - height of rectangle
-//              col - color
-// RETURNS    : -
-////////////////////////////////////////////////////////////////////////
-
-void Surface::Colorize( short x, short y, unsigned short w, unsigned short h,
-                        const Color &col ) const {
-  SDL_Surface *s = s_surface;
-  Sint32 px, py, ypnt;
-  Uint32 transp = MapRGB( GetColorKey() );
-  Uint32 color = MapRGB( col );
-
-  if ( SDL_MUSTLOCK( s ) && (SDL_LockSurface( s ) < 0) ) return;
-
-  switch ( s->format->BytesPerPixel ) {
-    case 1: {
-      Uint8 *pnt;
-      for ( py = y; py < y + h; ++py ) {
-        ypnt = py * s->pitch;
-        for ( px = x; px < x + w; ++px ) {
-          pnt = (Uint8 *)s->pixels + px + ypnt;
-          if ( *pnt != transp ) *pnt= (Uint8)color;
-        }
-      }
-      break;
-    }
-    case 2: {
-      Uint16 *pnt;
-      for ( py = y; py < y + h; ++py ) {
-        ypnt = py * s->pitch/2;
-        for ( px = x; px < x + w; ++px ) {
-          pnt = (Uint16 *)s->pixels + px + ypnt;
-          if ( *pnt != transp ) *pnt = (Uint16)color;
-        }
-      }
-      break;
-    }
-    case 3:     // 24-bpp mode not supported
-      break;
-
-    case 4: {
-      Uint32 *pnt;
-      for ( py = y; py < y + h; ++py ) {
-        ypnt = py * s->pitch/4;
-        for ( px = x; px < x + w; ++px ) {
-          pnt = (Uint32 *)s->pixels + px + ypnt;
-          if ( *pnt != transp ) *pnt= (Uint32)color;
-        }
-      } 
-      break;
-    }
-  }
-
-  if ( SDL_MUSTLOCK( s ) ) SDL_UnlockSurface( s );
-}
-
 ////////////////////////////////////////////////////////////////////////
 // NAME       : Surface::DrawBox
 // DESCRIPTION: Draw a bevelled 3-dimensional box.
@@ -506,15 +442,12 @@ int Surface::LoadPalette( File &file, unsigned short num ) {
 int Surface::LoadImageData( File &file, bool hwsurface /* = false */ ) {
   unsigned char bpp, flags;
   unsigned short width, height, colors;
-  unsigned long packed;
   Color ckey;
-  int rc = 0;
 
   width = file.Read16();
   height = file.Read16();
   bpp = file.Read8();   // nothing but 8 supported for now
   flags = file.Read8();
-  packed = file.Read32();
   colors = file.Read16();
 
   if ( flags & RAW_DATA_TRANSPARENT ) {
@@ -526,33 +459,15 @@ int Surface::LoadImageData( File &file, bool hwsurface /* = false */ ) {
   if ( Create( width, height, bpp, (hwsurface ? SDL_HWSURFACE : 0) ) ) return -1;
   if ( LoadPalette( file, colors ) ) return -1;
 
-  unsigned char *buf = new unsigned char[packed], *unbuf;
-  if ( !buf ) rc = -1;
-  else {
-    file.Read( buf, packed );
-
-    if ( packed != (unsigned long)(width * height) ) {   // data is compressed
-      unbuf = new unsigned char[width * height];
-      if ( !unbuf ) rc = -1;
-      else rc = unpack( unbuf, width * height, buf, packed );
-      delete [] buf;
-    } else unbuf = buf;
-
-    if ( !rc ) {
-      for ( int y = 0; y < height; ++y ) {
-        for ( int x = 0; x < width; ++x ) {
-          *((Uint8 *)s_surface->pixels + x + y * s_surface->pitch) =
-                                                unbuf[x + y * width];
-        }
-      }
+  for ( int y = 0; y < height; ++y ) {
+    if ( file.Read( (Uint8 *)s_surface->pixels + y * s_surface->pitch, width ) != width ) {
+      return -1;
     }
-
-    delete [] unbuf;
   }
 
   if ( flags & RAW_DATA_TRANSPARENT ) SetColorKey( ckey );
 
-  return rc;
+  return 0;
 }
 
 ////////////////////////////////////////////////////////////////////////
