@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -190,6 +190,32 @@ GUI_Status TextScrollWidget::WidgetActivated( Widget *widget, Window *win ) {
   return GUI_OK;
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : TextScrollWidget::MouseDown
+// DESCRIPTION: Distribute mouse down events to all components. Redirect
+//              all scrollwheel events to the slider so that scrolling
+//              works even if the pointer doesn't hover over the slider.
+// PARAMETERS : button - button event
+// RETURNS    : GUI status
+////////////////////////////////////////////////////////////////////////
+
+GUI_Status TextScrollWidget::MouseDown( const SDL_MouseButtonEvent &button ) {
+  SDL_MouseButtonEvent mybutton = button;
+  if ( ((button.button == SDL_BUTTON_WHEELUP) ||
+       (button.button == SDL_BUTTON_WHEELDOWN)) &&
+       Contains( button.x - surface->LeftEdge(), button.y - surface->TopEdge() ) ) {
+    SliderWidget *s = static_cast<SliderWidget *>( GetWidget(1) );
+    if ( s ) {
+      // to address the slider widget we must adjust the click
+      // coordinates to be inside the slider box
+      mybutton.x = s->LeftEdge() + surface->LeftEdge();
+      mybutton.y = s->TopEdge() + surface->TopEdge();
+    }
+  }
+
+  return CompositeWidget::MouseDown( mybutton );
+}
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : TextScrollWidget::SetText
 // DESCRIPTION: Set a new text to be displayed in the widget.
@@ -505,7 +531,7 @@ void StringWidget::CharInput( short sym, unsigned short unicode ) {
       if ( buffer.size() < maxlen ) {   // insert char at cursor pos
         bool accept = (validator ?
                        validator->ValidateKey( buffer.c_str(), unicode, cursor ) :
-                       (isalnum(unicode) && (unicode<=127)) );
+                       isprint(unicode) != 0 );
         if ( accept ) {
           buffer.insert( cursor++, 1, unicode );
           changed = true;
