@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -63,13 +63,7 @@ Window::Window( unsigned short flags, View *view ) {
 ////////////////////////////////////////////////////////////////////////
 
 Window::~Window( void ) {
-  Widget *wd = widgets;
-
-  while ( wd ) {
-    Widget *wd2 = wd->next;
-    delete wd;
-    wd = wd2;
-  }
+  RemoveAllWidgets();
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -204,21 +198,39 @@ void Window::AddWidget( Widget *widget ) {
 // NAME       : Window::RemoveWidget
 // DESCRIPTION: Remove a widget from the window.
 // PARAMETERS : widget - widget to remove
-// RETURNS    : pointer to the widget
+// RETURNS    : pointer to the widget removed
 ////////////////////////////////////////////////////////////////////////
 
 Widget *Window::RemoveWidget( Widget *widget ) {
   Widget *wd = widgets, **wdptr = &widgets;
 
-  while ( wd != widget ) {
+  while ( wd && wd != widget ) {
     wdptr = &wd->next;
     wd = wd->next;
   }
 
-  *wdptr = wd->next;
+  if ( wd ) *wdptr = wd->next;
   return wd;
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : Window::RemoveAllWidgets
+// DESCRIPTION: Remove all widgets from the window.
+// PARAMETERS : -
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void Window::RemoveAllWidgets( void ) {
+  Widget *wd = widgets;
+
+  while ( wd ) {
+    Widget *wd2 = wd->next;
+    delete wd;
+    wd = wd2;
+  }
+  widgets = NULL;
+}
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : Window::HandleEvent
 // DESCRIPTION: Distribute system events to the window widgets.
