@@ -1,7 +1,7 @@
 /*
 bi2cf - A map converter for Crimson Fields
 copyright (c) 2002      by Florian Dietrich
-              2003-2004 by Jens Granseuer
+              2003-2005 by Jens Granseuer
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -38,15 +38,15 @@ extern const tconvdata bidd2_convdata;
 extern const tconvdata hl_convdata;
 
 #ifdef WIN32
-const unsigned char *pathdelimiter = "\\";
+const char *pathdelimiter = "\\";
 #else
-const unsigned char *pathdelimiter = "/";
+const char *pathdelimiter = "/";
 #endif
 
-const unsigned char *mapstrings[5] = {"unknown","BI","BIDD1","BIDD2","HL"};
-const char verstr[] = "0.4.0";
+const char *mapstrings[5] = {"unknown","BI","BIDD1","BIDD2","HL"};
+const char verstr[] = "0.4.4";
 
-const unsigned char helpstr[] =
+const char helpstr[] =
 "Usage: %s mapname [options]\n"
 "\n"
 "(mapname must be the map filename without extension, e.g. \"00\")\n"
@@ -221,8 +221,7 @@ void putunit(FILE *myfile, tconvdata convdata, int xpos, int ypos, unsigned char
         fprintf(myfile,"type = %s\n",convdata.units[type]);
         fprintf(myfile,"player = %d\n",player);
         fprintf(myfile,"id = %d\n",id);
-        fprintf(myfile,"xpos = %d\n",xpos);
-        fprintf(myfile,"ypos = %d\n",ypos);
+        fprintf(myfile,"pos = %d/%d\n",xpos,ypos);
         fprintf(myfile,"\n");
     }
 }
@@ -265,8 +264,7 @@ void putbuilding(FILE *myfile, tconvdata convdata, tmapdata mapdata, int xpos, i
     fprintf(myfile,"crystals = %d\n",crystals);
     fprintf(myfile,"player = %i\n",(type+1)%3);
     fprintf(myfile,"id = %d\n",id);
-    fprintf(myfile,"xpos = %d\n",xpos);
-    fprintf(myfile,"ypos = %d\n",ypos);
+    fprintf(myfile,"pos = %d/%d\n",xpos,ypos);
     fprintf(myfile,"\n");
 }
 
@@ -452,7 +450,7 @@ void write_events(FILE *myfile, int hqid[])
         fprintf(myfile,"trigger = havebuilding\n");
         fprintf(myfile,"tbuilding = %d\n",hqid[2-i]);
         fprintf(myfile,"towner = %d\n", i);
-        fprintf(myfile,"tturn = -1\n");
+        fprintf(myfile,"ttime = -1\n");
         fprintf(myfile,"success = 100\n\n");
 
         // player i destroys all enemy units
@@ -469,8 +467,8 @@ void write_events(FILE *myfile, int hqid[])
         fprintf(myfile,"id = %d\n", id++);
         fprintf(myfile,"type = message\n");
         fprintf(myfile,"player = %d\n", i);
-        fprintf(myfile,"trigger = turn\n");
-        fprintf(myfile,"tturn = 1\n");
+        fprintf(myfile,"trigger = timer\n");
+        fprintf(myfile,"ttime = 0\n");
         fprintf(myfile,"message = 0\n\n");
     }
 }
